using System;
using System.Runtime.InteropServices;
using System.Collections;
using Scenario.WorkflowEngine.DataModel.WorkflowApplicationsData;
using Scenario.WorkflowEngine.DataModel.Common;

namespace Scenario.Applications
{
		//callback function for executing Application API
		public delegate String CallBack( String sValue);

		public class ApplicationHelper
		{
			IntPtr instance;
			ArrayList vCallBacks;

			public ApplicationHelper()
			{
                instance = ApplicationHelper.CreateClass();
				vCallBacks = new ArrayList();
			}

			public void Dispose()
			{
                ApplicationHelper.DeleteClass(instance);
			}

			public void StartUp(String sApplicationName)
			{
                ApplicationHelper.StartUp(instance, sApplicationName);
			}

			public void AddCallBack(CallBack cb, String sPrimitiveName)
			{
				/*
				 * Il vettore delle callback  utilizzato per memorizzare le callback in modo
				 * da evitare che la garbage collector di .NET cancelli il riferimento al puntatore.
				 * Il puntatore della callback viene creato e passato alla libreria ApplicationTemplate.
				 * Dato che il passaggio avviene grazie all'interoperabilit il compilatore .NET cancella il riferimento
				 * in quanto non tiene traccia che quest'ultimo  ancora utilizzato.Per evitarlo questa condizione
				 * deve esistere una variabile che memorizza il riferimento alla callback per tutta la durata dell'esecuzione
				 * dell'application. La lista  utile in quanto esoner lo sviluppatore dell'application da quest'onere.
				*/
				vCallBacks.Add ( cb );
                ApplicationHelper.AddCallBack(instance, cb, sPrimitiveName);
			}

			public static void ExploreSignature(string sFunction,out XMLApplicationFunction oXMLApplicationFunction, out Hashtable FormalParameterIn,out Hashtable FormalParameterOut)
			{
				// Initializes the out par
				FormalParameterIn = new Hashtable();
				FormalParameterOut = new Hashtable();

				oXMLApplicationFunction = new XMLApplicationFunction();
			
				//build the out oXMLApplicationFunction
				oXMLApplicationFunction.Parse(sFunction);

				Console.WriteLine( "\nCallback called with param: {0}", sFunction );	
					
				// Interroga la lista dei parametri di ingresso
				for ( int i=0; i<oXMLApplicationFunction.XMLFormalParameters.Count; i++)
				{
					XMLFormalParameter oXMLFormalParameter = new XMLFormalParameter();
					oXMLFormalParameter=(XMLFormalParameter)oXMLApplicationFunction.XMLFormalParameters[i];
				
					//build the hashtable of signature (pramName,value)
					FormalParameterIn.Add(oXMLFormalParameter.Name, oXMLFormalParameter.Value);
				}

				for ( int i=0; i<oXMLApplicationFunction.XMLApplicationFunctionResult.XMLFormalParameters.Count; i++)
				{
					XMLFormalParameter oXMLFormalParameter = new XMLFormalParameter();
					oXMLFormalParameter=(XMLFormalParameter)oXMLApplicationFunction.XMLApplicationFunctionResult.XMLFormalParameters[i];
				
					//build the hashtable of signature (pramName,value)
					FormalParameterOut.Add(oXMLFormalParameter.Name, oXMLFormalParameter.Value);
				}

			}

			public static string ComposeResponse( XMLApplicationFunction oXMLApplicationFunction,int code, string message, Hashtable FormalParameterOut)
			{
				oXMLApplicationFunction.XMLApplicationFunctionResult.Code=code;
				oXMLApplicationFunction.XMLApplicationFunctionResult.Message=message;

				for ( int i=0; i<oXMLApplicationFunction.XMLApplicationFunctionResult.XMLFormalParameters.Count; i++)
				{
					XMLFormalParameter oXMLFormalParameter = new XMLFormalParameter();
					oXMLFormalParameter=(XMLFormalParameter)oXMLApplicationFunction.XMLApplicationFunctionResult.XMLFormalParameters[i];
					//Add value to out parameter
					oXMLFormalParameter.Value=(string)FormalParameterOut[oXMLFormalParameter.Name];

				}

				string resultMain =  oXMLApplicationFunction.XMLApplicationFunctionResult.GetXML();
				System.Console.WriteLine ("{0} {1}", "OUT :", resultMain);
				return resultMain;
			}


            [DllImport("Criteria.Applications.ApplicationHelper.dll")]
			private static extern IntPtr CreateClass();

            [DllImport("Criteria.Applications.ApplicationHelper.dll")]
			private static extern void DeleteClass( IntPtr instance );

            [DllImport("Criteria.Applications.ApplicationHelper.dll")]
			private static extern void AddCallBack( IntPtr instance, CallBack cb, String sPrimitiveName );

            [DllImport("Criteria.Applications.ApplicationHelper.dll")]
			private static extern void StartUp( IntPtr instance, string sApplicationName );

			#region Static Methods used before DataModel.NET develpment. Now They are commented.
			/*
			[ DllImport( "ApplicationTemplate.dll" )]
			private static extern void GetFunctionParameters( string sFunction, out int size, out IntPtr outArray );	

			[ DllImport( "ApplicationTemplate.dll" )]
			public static extern string GetFunctionResult( string sMessage, int iCode, int iSize, out IntPtr  outArray );
			*/

			/*
			public static ArrayList GetInputParamter(String sFunction)	
			{
				int size;
				IntPtr outArray;
			
				ApplicationInterface.GetFunctionParameters( sFunction, out size, out outArray );
			
				FunctionParameter parameter;
				ArrayList vParameters = new ArrayList();
				IntPtr current = outArray;
				for( int i = 0; i < size; i++ )
				{
					parameter = new FunctionParameter();
					Marshal.PtrToStructure( current, parameter);
				
					//Marshal.FreeCoTaskMem( (IntPtr)Marshal.ReadInt32( current ));
					Marshal.DestroyStructure( current, typeof(FunctionParameter) );

					current = (IntPtr)((int)current + Marshal.SizeOf( parameter));
				
					vParameters.Add ( parameter);
				}
				Marshal.FreeCoTaskMem( outArray );	

				return vParameters;
			}
			*/
			#endregion

		}

		#region FunctionResult: Used before DataModel.NET developed. Now it's commented.
		/*
		
		//Parameter for Application API
		[ StructLayout( LayoutKind.Sequential, CharSet=CharSet.Ansi )]
		public class FunctionParameter 
		{
			public String sName;
			public String sValue;
		}

		public class FunctionResult
		{
			public int iCode;
			public String sMessage;

			private IntPtr instance;

			public FunctionResult(String sMessage, int iCode)
			{
				vParameters = new ArrayList();
				instance = FunctionResult.CreateResultClass (sMessage, iCode);
			}

			public void Dispose()
			{
				FunctionResult.DeleteResultClass (instance);
			}

			public void AddParamter(FunctionParameter parameter)
			{
				FunctionResult.AddResultParameter (instance, parameter.sName , parameter.sValue);
			}

			public void AddParamter(String sName, String sValue)
			{
				FunctionResult.AddResultParameter (instance, sName, sValue);
			}

			public void AddXmlParameter(String sName, String sValue)
			{
				//TODO: Sostituire STRING con l'attributo statico.
				FunctionResult.AddResultParameterType(instance, sName, sValue, "XML");
			}

			public String GetXml()
			{
				return FunctionResult.GetResultXml (instance);
			}
	 
			private ArrayList vParameters;

			[ DllImport( "ApplicationTemplate.dll" )]
			private static extern IntPtr CreateResultClass(string sMessage, int iCode);	
		
			[ DllImport( "ApplicationTemplate.dll" )]
			private static extern void DeleteResultClass( IntPtr instance );

			[ DllImport( "ApplicationTemplate.dll" )]
			private static extern void AddResultParameter( IntPtr instance, String sName, String sValue );

			[ DllImport( "ApplicationTemplate.dll" )]
			private static extern void AddResultParameterType( IntPtr instance, String sName, String sValue, String sType );

			[ DllImport( "ApplicationTemplate.dll" )]
			private static extern string GetResultXml(IntPtr instance);
		}
		*/
		#endregion
}
