using System;
using System.Text;
using System.Data;
using System.Configuration;

using System.Collections;
using System.Collections.Specialized;

using MySql.Data.MySqlClient;

namespace bOS.DataBase
{
	#region DataBase Manager Class Exception
	public class DataBaseManagerException : System.Exception 
	{
		public DataBaseManagerException(string sError) : base(sError)
		{
		}
	}

	public class DataBaseArgumentException : DataBaseManagerException 
	{
		public DataBaseArgumentException(string sError) : base(sError)
		{
		}
	}

	#endregion

	public class DataBaseManager
	{
		protected string sConnectionString = string.Empty;
		
		#region Property methods
		public string ConnectionString 
		{
			get {return sConnectionString;}
			set {sConnectionString = value;}
		}
		#endregion
		
		public DataBaseManager()
		{
		}

		public DataBaseManager(string _ConnectionString)
		{
			this.ConnectionString = _ConnectionString;
		}


		#region General Execution methods

		private int RetrieveLastID(ref MySqlConnection mySqlConnection)
		{
			MySqlCommand mySQLCommand = null;
			int iID =0;
			string sSQL = "select LAST_INSERT_ID()";
			try
			{
				mySQLCommand = new MySqlCommand (sSQL);
				mySQLCommand.Connection = mySqlConnection;
				MySqlDataReader returnData = mySQLCommand.ExecuteReader ();

				while (returnData.Read()) 
				{
					iID = returnData.GetInt32 (0);
				}
			}
            catch (MySql.Data.MySqlClient.MySqlException err)
			{
				string sError = err.Message;
				sError+=sSQL;
			}

			return iID;
		}

		//utile per le select che restituiscono un solo campo. Esempio:
		//select count(*) from ....
		public object ExecuteScalar(string sQuery2Execute) 
		{
			if (sQuery2Execute==null || (sQuery2Execute.CompareTo (string.Empty ) == 0 ) )
			{
				throw (new DataBaseArgumentException("sQuery2Execute is null or empty"));
			}
			MySqlCommand myCommand = new MySqlCommand(sQuery2Execute);

			return this.ExecuteScalar (myCommand);
		}

		public object ExecuteScalar(MySqlCommand myCommand) 
		{
			MySqlConnection myConnection = new MySqlConnection(sConnectionString);
			myCommand.Connection = myConnection;
			myConnection.Open();
			object result = myCommand.ExecuteScalar();
			myCommand.Connection.Close();

			return result;
		}


		//utile per eseguire insert, upadate e delete. Il metodo restituisce il numero
		//di record inseriti, modificati, cancellati.
		public int ExecuteNoQuery(string sQuery2Execute) 
		{
			if (sQuery2Execute==null || (sQuery2Execute.CompareTo (string.Empty ) == 0 ) )
			{
				throw (new DataBaseArgumentException("sQuery2Execute is null or empty"));
			}

			MySqlCommand myCommand = new MySqlCommand(sQuery2Execute);

			return this.ExecuteNoQuery (myCommand);
		}

		public int ExecuteAddQuery(MySqlCommand myCommand) 
		{
			MySqlConnection myConnection = new MySqlConnection(sConnectionString);
			myCommand.Connection = myConnection;
			myConnection.Open();

			int result = 0;

			try
			{
				result = myCommand.ExecuteNonQuery ();
			}
            catch (MySql.Data.MySqlClient.MySqlException myErr)
			{
				throw new DataBaseManagerException(myErr.Message);
			}

			if ( result == 0 )
				return 0;

			int iIdRow = this.RetrieveLastID(ref myConnection);

			myCommand.Connection.Close();

			return iIdRow;

		}


		

		public int ExecuteNoQuery(MySqlCommand myCommand) 
		{
			MySqlConnection myConnection = new MySqlConnection(sConnectionString);
			myCommand.Connection = myConnection;
			myConnection.Open();

			int result = 0;

			try
			{
				result = myCommand.ExecuteNonQuery ();
			}
            catch (MySql.Data.MySqlClient.MySqlException myErr)
			{
				throw new DataBaseManagerException(myErr.Message);
			}

			myCommand.Connection.Close();

			return result;

		}

		//solo per compatibilit con i vecchi progetti.
		public MySqlCommand GetCommand(string sSQL)
		{
			MySqlCommand myCommand = new MySqlCommand (sSQL);
			MySqlConnection myConnection = new MySqlConnection(sConnectionString);
			myCommand.Connection = myConnection;
			myConnection.Open();

			return myCommand;

		}

		public DataView GetDataView(String sSQL)
		{
			MySqlCommand myCommand = new MySqlCommand (sSQL);
			MySqlConnection myConnection = new MySqlConnection(sConnectionString);
			myCommand.Connection = myConnection;
			myConnection.Open();

			DataTable dt = new DataTable();
			MySqlDataReader reader = myCommand.ExecuteReader();
		
			for ( int i=0; i<reader.FieldCount;i++)
			{
				dt.Columns.Add(new DataColumn(reader.GetName(i), reader.GetFieldType(i)));
			}
			DataRow dr = null;
			while ( reader.Read() )
			{
				dr = dt.NewRow();
				for ( int i=0; i<reader.FieldCount;i++)
				{
					dr[i] = reader.GetValue(i);
				}
				dt.Rows.Add(dr);
			}
			reader.Close();

			myCommand.Connection.Close();

			return new DataView(dt);
		}
		
		
		public void ReadFromDataBase (MySqlCommand myCommand, ref bOSTable objTable)
		{
								
			MySqlConnection myConnection = new MySqlConnection(sConnectionString);
			myCommand.Connection = myConnection;
			myConnection.Open();
			MySqlDataReader mySqlReader = myCommand.ExecuteReader ();
			objTable.ReadFromDataBase ( mySqlReader);
			myCommand.Connection.Close();
		}

		public void ReadFromDataBase (string sSelect2Execute, ref bOSTable objTable)
		{
			if (sSelect2Execute==null || (sSelect2Execute.CompareTo (string.Empty ) == 0 ) )
			{
				throw (new DataBaseArgumentException("sSelect2Execute is null or empty"));
			}
					
			MySqlCommand myCommand = new MySqlCommand(sSelect2Execute);

			this.ReadFromDataBase ( myCommand, ref objTable);
		}


		public SelectionValuesCollection GetColletionFromQuery(string sQuery2Execute) 
		{
			if (sQuery2Execute == null || sQuery2Execute.Length==0)
				throw (new DataBaseArgumentException("sQuery2Execute is null or empty"));

			try
			{
				bOS.DataBase.bOSTable boRecords = new bOS.DataBase.SelectionValuesCollection();

				this.ReadFromDataBase ( sQuery2Execute, ref boRecords);

				return (bOS.DataBase.SelectionValuesCollection) boRecords;

			}
            catch (MySql.Data.MySqlClient.MySqlException err)
			{
				string sError = err.Message;
				sError+=sQuery2Execute;

				throw new DataBaseManagerException (sError);

			}

		}
		
		public string[,] GetFields(string sSQL)
		{
			MySqlCommand sqlCmd = new MySqlCommand (sSQL);
			MySqlConnection myConnection = new MySqlConnection(sConnectionString);
			sqlCmd.Connection = myConnection;
			myConnection.Open();

			MySqlDataReader reader = sqlCmd.ExecuteReader ();

			String[,] vFields = null;
			if ( reader.Read () )
			{
				int iRow = reader.FieldCount;

				vFields = new String[iRow,2];

				for ( int i=0;i<reader.FieldCount;i++)
				{
					vFields[i,0] = reader.GetName(i);
					vFields[i,1] = reader.GetValue(i).ToString();
				}
			}
			myConnection.Close();

			return vFields;
		}


		#endregion
	}
}

