using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using System.Reflection;
using System.Configuration;


using WorkflowProcesses = Scenario.WorkflowEngine.DataModel.WorkflowProcesses;
using WorkflowRelevantData = Scenario.WorkflowEngine.DataModel.WorkflowRelevantData;
using WorkflowParticipantsData = Scenario.WorkflowEngine.DataModel.WorkflowParticipantsData;

using Visio = Microsoft.Office.Interop.Visio;
using log4net;
using log4net.Config;

namespace Scenario.Modeler
{
	/// <summary>
	/// Descrizione di riepilogo per MainForm.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		// Constants
		private const string COST_PROPERTY = "Cost";
		private const string AUDIT_DIAGRAM_TEMPLATE = "BPMN.VST";
		private const string CUSTOM_PROPERTY_PREFIX = "Prop.";
		private const string ASTERISK = "*";

		private const string Start_MASTER = "Start";
		private const string End_MASTER = "Intermediate";
		private const string Intermediate_MASTER = "Master";
		private const string Activity_MASTER = "Activity";
		private const string Gateway_MASTER = "Gateway";
		private const string GatewayOR_MASTER = "Gateway OR";
		private const string GatewayAND_MASTER = "Gateway AND";
		private const string GatewayXOR_MASTER = "Gateway XOR";
		private const string GatewayEvent_MASTER = "Gateway Event-Based";
		private const string GatewayComplex = "Gateway Complex";

		private const string Flow_MASTER = "Flow";
        private const string MessageFlow_MASTER = "Message Flow";
		//private const string Attribute_MASTER = "Data Object";
		private const string Attribute_MASTER = "3D stack top";
		private const string COLLAPSED_SUBPROCESS_MASTER = "Collapsed Sub-Process";

		private Visio.Document auditDocument;
		private Visio.Pages pagesInDocument;
		
		// Variables
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		private System.Windows.Forms.ContextMenu shapeShortcutMenu;
		private System.Windows.Forms.MenuItem shortcutMenu;
		private System.Windows.Forms.SaveFileDialog saveFileDialog1;
		private System.Windows.Forms.MenuItem menuItem8;
		private System.Windows.Forms.MenuItem menuItem11;
		private System.Windows.Forms.MenuItem menuItem12;
		private System.Windows.Forms.MenuItem menuItem13;
		private System.Windows.Forms.MenuItem menuItem14;
		private System.Windows.Forms.MenuItem menuItem17;
		private System.Windows.Forms.MenuItem menuItem21;
		private System.Windows.Forms.MenuItem menuItem22;
		private System.Windows.Forms.MenuItem menuCheck;
		private System.Windows.Forms.MenuItem menuTutorial;
		private System.Windows.Forms.MenuItem menuManual;
		private System.Windows.Forms.MenuItem menuAboutScenario;
		private System.Windows.Forms.MenuItem menuAboutModeler;
		private System.Windows.Forms.MenuItem menuAboutProAndPro;
		private System.Windows.Forms.MenuItem menuItem15;
		private System.Windows.Forms.MenuItem menuWWWLinks;
		private System.Windows.Forms.MenuItem menuItem18;
		private System.Windows.Forms.MenuItem menuNew;
		private System.Windows.Forms.MenuItem menuOpen;
		private System.Windows.Forms.MenuItem menuSave;
		private System.Windows.Forms.MenuItem menuSaveAs;
		private System.Windows.Forms.MenuItem menuConfiguration;
		private System.Windows.Forms.MenuItem menuRecentFiles;
		private System.Windows.Forms.MenuItem menuExit;
		private System.Windows.Forms.MenuItem menuApplications;
		private System.Windows.Forms.MenuItem menuDataTypes;
		private System.Windows.Forms.MenuItem menuPartecipants;
		private System.Windows.Forms.MenuItem menuManageCheck;
		private System.Windows.Forms.MenuItem menuProcessProperties;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuItem4;
        private System.Windows.Forms.MenuItem mnuRotateCanvas;
        private IContainer components;
		private System.Windows.Forms.MenuItem mnuCreateTemplate;
		private System.Windows.Forms.MenuItem mnuStartTemplate;
		private System.Windows.Forms.MenuItem mnuPrintPreview;
		private System.Windows.Forms.MenuItem menuSaveasWebPage;
		private System.Windows.Forms.MenuItem menuItem7;
		private System.Windows.Forms.MenuItem menuStencilInfosName;
		private System.Windows.Forms.MenuItem mnuWindowSettings;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem menuItem10;
		private System.Windows.Forms.MenuItem menuItem16;
		private System.Windows.Forms.MenuItem menuItem19;
		private System.Windows.Forms.MenuItem menuLayersManager;
		private System.Windows.Forms.MenuItem menuItem20;

		private string sScenarioExecutionPath = string.Empty;

		/// <summary>
		/// Utilizzate per collezionare i partecipanti ai workflow disegnati.
		/// </summary>
		private WorkflowParticipantsData.XMLRoles xmlRoles;
		private System.Windows.Forms.FolderBrowserDialog folderBrowserDialog1;
		private System.Windows.Forms.MenuItem menuItem23;
		private System.Windows.Forms.MenuItem mnuXSDEditor;
		private System.Windows.Forms.MenuItem menuXMLDB;
		private System.Windows.Forms.MenuItem menuXSDInference;
		private System.Windows.Forms.MenuItem menuNewBP;
		private System.Windows.Forms.MenuItem menuNewPCM;
		private System.Windows.Forms.MenuItem mnuSendFeedBack;
		private System.Windows.Forms.MenuItem menuItem25;

		/// <summary>
		/// Utilizzato per collezionare i workflow associati ai singoli fogli.
		/// E' indispensabile collezionare workflow in quanto non tutte le infos
		/// valorizzabile con il modeler sono associate a shapes e quindi vengono
		/// memorizzare direttamente sul workflow.
		/// </summary>
		public WorkflowProcesses.XMLWorkflows xmlWorkflows;
		private System.Windows.Forms.MenuItem menuInstanceOpen;
		private System.Windows.Forms.MenuItem menuInstanceRestart;
		private System.Windows.Forms.MenuItem menuItem26;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.MenuItem menuItem6;

		bool bNotApplyAddEvent = false;
        private AxMicrosoft.Office.Interop.VisOcx.AxDrawingControl axDrawingControl1;
        private ToolStrip toolStrip1;

        private ToolStripSeparator toolStripSeparator1;
        private ToolStripButton toolStripButton1;
        private ToolStripButton toolStripButton3;
        private ToolStripButton toolStripButton2;
        private ToolStripButton toolStripPartecipants;
        private ToolStripButton toolStripButton5;
        private ToolStripButton toolStripButton6;
        private ToolStripSeparator toolStripSeparator2;
        private ToolStripButton toolStripButton7;
        private ToolStripSeparator toolStripSeparator3;
        private ToolStripSplitButton toolStripButton8;
        private ToolStripMenuItem creatToolStripMenuItem;
        private ToolStripMenuItem eseguiToolStripMenuItem;
        private ToolStripMenuItem apriToolStripMenuItem;
        private ToolStripMenuItem restartToolStripMenuItem;
        private ToolStripButton toolStripButton4;


		private static readonly ILog log = LogManager.GetLogger(typeof(MainForm));

		public MainForm()
		{
			string sConfigFile = ((Assembly.GetEntryAssembly()).GetName()).Name+
																".exe.config";

			XmlConfigurator.Configure(new System.IO.FileInfo(sConfigFile));

			//
			// Necessario per il supporto di Progettazione Windows Form
			//
			InitializeComponent();

			log.Info("Inizializing Scenario Engine");
			//sScenarioExecutionPath = WorkflowEngine.Engine.FolderHelper.GetFolder(WAPI.FolderHelper.FolderType.Execution);
			this.OpenDocument ("D:\\mySoftware\\ScenarioFramework\\TestEnvironment\\blank.vsd");
            //Visio.Document document = Utility.OpenStencil(this.axDrawingControl1, "C:\\mysoftware\\Scenario\\src\\3visio\\Frapu-BPMN_Template.vss");

			/*
			http://www.kbalertz.com/839079/Decreased.performance.shapes.contained.Visio.drawing.hosted.Visio.ActiveX.Control.aspx
			*/
			this.axDrawingControl1.Window.Application.OnDataChangeDelay = -2;
		}

		
		
		
		/// <summary>
		/// Pulire le risorse in uso.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Codice generato da Progettazione Windows Form
		/// <summary>
		/// Metodo necessario per il supporto della finestra di progettazione. Non modificare
		/// il contenuto del metodo con l'editor di codice.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(MainForm));
            this.mainMenu1 = new System.Windows.Forms.MainMenu(this.components);
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuNew = new System.Windows.Forms.MenuItem();
            this.menuNewBP = new System.Windows.Forms.MenuItem();
            this.menuNewPCM = new System.Windows.Forms.MenuItem();
            this.menuOpen = new System.Windows.Forms.MenuItem();
            this.menuItem8 = new System.Windows.Forms.MenuItem();
            this.menuSave = new System.Windows.Forms.MenuItem();
            this.menuSaveAs = new System.Windows.Forms.MenuItem();
            this.menuSaveasWebPage = new System.Windows.Forms.MenuItem();
            this.menuItem11 = new System.Windows.Forms.MenuItem();
            this.mnuPrintPreview = new System.Windows.Forms.MenuItem();
            this.menuItem9 = new System.Windows.Forms.MenuItem();
            this.menuConfiguration = new System.Windows.Forms.MenuItem();
            this.menuItem12 = new System.Windows.Forms.MenuItem();
            this.menuRecentFiles = new System.Windows.Forms.MenuItem();
            this.menuItem13 = new System.Windows.Forms.MenuItem();
            this.menuExit = new System.Windows.Forms.MenuItem();
            this.menuItem21 = new System.Windows.Forms.MenuItem();
            this.menuProcessProperties = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuCheck = new System.Windows.Forms.MenuItem();
            this.menuManageCheck = new System.Windows.Forms.MenuItem();
            this.menuItem22 = new System.Windows.Forms.MenuItem();
            this.menuApplications = new System.Windows.Forms.MenuItem();
            this.menuDataTypes = new System.Windows.Forms.MenuItem();
            this.menuPartecipants = new System.Windows.Forms.MenuItem();
            this.menuItem26 = new System.Windows.Forms.MenuItem();
            this.mnuCreateTemplate = new System.Windows.Forms.MenuItem();
            this.menuItem5 = new System.Windows.Forms.MenuItem();
            this.mnuStartTemplate = new System.Windows.Forms.MenuItem();
            this.menuItem6 = new System.Windows.Forms.MenuItem();
            this.menuInstanceOpen = new System.Windows.Forms.MenuItem();
            this.menuInstanceRestart = new System.Windows.Forms.MenuItem();
            this.menuItem4 = new System.Windows.Forms.MenuItem();
            this.mnuRotateCanvas = new System.Windows.Forms.MenuItem();
            this.menuLayersManager = new System.Windows.Forms.MenuItem();
            this.menuItem14 = new System.Windows.Forms.MenuItem();
            this.menuManual = new System.Windows.Forms.MenuItem();
            this.menuTutorial = new System.Windows.Forms.MenuItem();
            this.menuItem17 = new System.Windows.Forms.MenuItem();
            this.mnuSendFeedBack = new System.Windows.Forms.MenuItem();
            this.menuItem25 = new System.Windows.Forms.MenuItem();
            this.menuAboutModeler = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuItem15 = new System.Windows.Forms.MenuItem();
            this.menuAboutScenario = new System.Windows.Forms.MenuItem();
            this.menuAboutProAndPro = new System.Windows.Forms.MenuItem();
            this.menuItem18 = new System.Windows.Forms.MenuItem();
            this.menuWWWLinks = new System.Windows.Forms.MenuItem();
            this.menuItem7 = new System.Windows.Forms.MenuItem();
            this.menuStencilInfosName = new System.Windows.Forms.MenuItem();
            this.mnuWindowSettings = new System.Windows.Forms.MenuItem();
            this.menuItem10 = new System.Windows.Forms.MenuItem();
            this.menuItem16 = new System.Windows.Forms.MenuItem();
            this.menuItem19 = new System.Windows.Forms.MenuItem();
            this.menuItem20 = new System.Windows.Forms.MenuItem();
            this.menuItem23 = new System.Windows.Forms.MenuItem();
            this.mnuXSDEditor = new System.Windows.Forms.MenuItem();
            this.menuXMLDB = new System.Windows.Forms.MenuItem();
            this.menuXSDInference = new System.Windows.Forms.MenuItem();
            this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
            this.shapeShortcutMenu = new System.Windows.Forms.ContextMenu();
            this.shortcutMenu = new System.Windows.Forms.MenuItem();
            this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
            this.folderBrowserDialog1 = new System.Windows.Forms.FolderBrowserDialog();
            this.axDrawingControl1 = new AxMicrosoft.Office.Interop.VisOcx.AxDrawingControl();
            this.toolStrip1 = new System.Windows.Forms.ToolStrip();
            this.toolStripButton1 = new System.Windows.Forms.ToolStripButton();
            this.toolStripButton3 = new System.Windows.Forms.ToolStripButton();
            this.toolStripButton2 = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator1 = new System.Windows.Forms.ToolStripSeparator();
            this.toolStripButton4 = new System.Windows.Forms.ToolStripButton();
            this.toolStripPartecipants = new System.Windows.Forms.ToolStripButton();
            this.toolStripButton5 = new System.Windows.Forms.ToolStripButton();
            this.toolStripButton6 = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator2 = new System.Windows.Forms.ToolStripSeparator();
            this.toolStripButton7 = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator3 = new System.Windows.Forms.ToolStripSeparator();
            this.toolStripButton8 = new System.Windows.Forms.ToolStripSplitButton();
            this.creatToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.eseguiToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.apriToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.restartToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            ((System.ComponentModel.ISupportInitialize)(this.axDrawingControl1)).BeginInit();
            this.toolStrip1.SuspendLayout();
            this.SuspendLayout();
            // 
            // mainMenu1
            // 
            this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem1,
            this.menuItem21,
            this.menuItem26,
            this.menuItem4,
            this.menuItem14,
            this.menuItem7});
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 0;
            this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuNew,
            this.menuOpen,
            this.menuItem8,
            this.menuSave,
            this.menuSaveAs,
            this.menuSaveasWebPage,
            this.menuItem11,
            this.mnuPrintPreview,
            this.menuItem9,
            this.menuConfiguration,
            this.menuItem12,
            this.menuRecentFiles,
            this.menuItem13,
            this.menuExit});
            this.menuItem1.Text = "File";
            // 
            // menuNew
            // 
            this.menuNew.Index = 0;
            this.menuNew.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuNewBP,
            this.menuNewPCM});
            this.menuNew.Text = "New";
            this.menuNew.Click += new System.EventHandler(this.menuNew_Click);
            // 
            // menuNewBP
            // 
            this.menuNewBP.Index = 0;
            this.menuNewBP.Text = "Scenario Business Process";
            this.menuNewBP.Click += new System.EventHandler(this.menuNewBP_Click);
            // 
            // menuNewPCM
            // 
            this.menuNewPCM.Index = 1;
            this.menuNewPCM.Text = "Scenario PCM Process";
            this.menuNewPCM.Click += new System.EventHandler(this.menuNewPCM_Click);
            // 
            // menuOpen
            // 
            this.menuOpen.Index = 1;
            this.menuOpen.Text = "Open";
            this.menuOpen.Click += new System.EventHandler(this.mnuOpen_Click);
            // 
            // menuItem8
            // 
            this.menuItem8.Index = 2;
            this.menuItem8.Text = "-";
            // 
            // menuSave
            // 
            this.menuSave.Index = 3;
            this.menuSave.Text = "Save";
            this.menuSave.Click += new System.EventHandler(this.menuSave_Click);
            // 
            // menuSaveAs
            // 
            this.menuSaveAs.Index = 4;
            this.menuSaveAs.Text = "Save as";
            this.menuSaveAs.Click += new System.EventHandler(this.mnuSaveAs_Click);
            // 
            // menuSaveasWebPage
            // 
            this.menuSaveasWebPage.Index = 5;
            this.menuSaveasWebPage.Text = "Save as WEB Page";
            this.menuSaveasWebPage.Click += new System.EventHandler(this.menuSaveasWebPage_Click);
            // 
            // menuItem11
            // 
            this.menuItem11.Index = 6;
            this.menuItem11.Text = "-";
            // 
            // mnuPrintPreview
            // 
            this.mnuPrintPreview.Index = 7;
            this.mnuPrintPreview.Text = "Print Preview";
            this.mnuPrintPreview.Click += new System.EventHandler(this.mnuPrintPreview_Click);
            // 
            // menuItem9
            // 
            this.menuItem9.Index = 8;
            this.menuItem9.Text = "-";
            // 
            // menuConfiguration
            // 
            this.menuConfiguration.Index = 9;
            this.menuConfiguration.Text = "Configuration";
            this.menuConfiguration.Click += new System.EventHandler(this.menuConfiguration_Click);
            // 
            // menuItem12
            // 
            this.menuItem12.Index = 10;
            this.menuItem12.Text = "-";
            // 
            // menuRecentFiles
            // 
            this.menuRecentFiles.Index = 11;
            this.menuRecentFiles.Text = "Recent Files";
            this.menuRecentFiles.Click += new System.EventHandler(this.menuRecentFiles_Click);
            // 
            // menuItem13
            // 
            this.menuItem13.Index = 12;
            this.menuItem13.Text = "-";
            // 
            // menuExit
            // 
            this.menuExit.Index = 13;
            this.menuExit.Text = "Exit";
            this.menuExit.Click += new System.EventHandler(this.menuExit_Click);
            // 
            // menuItem21
            // 
            this.menuItem21.Index = 1;
            this.menuItem21.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuProcessProperties,
            this.menuItem3,
            this.menuCheck,
            this.menuManageCheck,
            this.menuItem22,
            this.menuApplications,
            this.menuDataTypes,
            this.menuPartecipants});
            this.menuItem21.Text = "Scenario Business Process";
            // 
            // menuProcessProperties
            // 
            this.menuProcessProperties.Index = 0;
            this.menuProcessProperties.Text = "Properties";
            this.menuProcessProperties.Click += new System.EventHandler(this.menuProcessProperties_Click);
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 1;
            this.menuItem3.Text = "-";
            // 
            // menuCheck
            // 
            this.menuCheck.Index = 2;
            this.menuCheck.Text = "Check";
            this.menuCheck.Click += new System.EventHandler(this.menuCheck_Click);
            // 
            // menuManageCheck
            // 
            this.menuManageCheck.Index = 3;
            this.menuManageCheck.Text = "Manage Check";
            this.menuManageCheck.Click += new System.EventHandler(this.menuManageCheck_Click);
            // 
            // menuItem22
            // 
            this.menuItem22.Index = 4;
            this.menuItem22.Text = "-";
            // 
            // menuApplications
            // 
            this.menuApplications.Index = 5;
            this.menuApplications.Text = "Applications";
            this.menuApplications.Click += new System.EventHandler(this.menuApplications_Click);
            // 
            // menuDataTypes
            // 
            this.menuDataTypes.Index = 6;
            this.menuDataTypes.Text = "DataTypes";
            this.menuDataTypes.Click += new System.EventHandler(this.menuDataTypes_Click);
            // 
            // menuPartecipants
            // 
            this.menuPartecipants.Index = 7;
            this.menuPartecipants.Text = "Partecipants";
            this.menuPartecipants.Click += new System.EventHandler(this.menuPartecipants_Click);
            // 
            // menuItem26
            // 
            this.menuItem26.Index = 2;
            this.menuItem26.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuCreateTemplate,
            this.menuItem5,
            this.mnuStartTemplate,
            this.menuItem6,
            this.menuInstanceOpen,
            this.menuInstanceRestart});
            this.menuItem26.Text = "Scenario Workflow Manager";
            // 
            // mnuCreateTemplate
            // 
            this.mnuCreateTemplate.Index = 0;
            this.mnuCreateTemplate.Text = "Create Template";
            this.mnuCreateTemplate.Click += new System.EventHandler(this.mnuCreateTemplate_Click);
            // 
            // menuItem5
            // 
            this.menuItem5.Index = 1;
            this.menuItem5.Text = "-";
            // 
            // mnuStartTemplate
            // 
            this.mnuStartTemplate.Index = 2;
            this.mnuStartTemplate.Text = "Start Template";
            this.mnuStartTemplate.Click += new System.EventHandler(this.mnuStartTemplate_Click);
            // 
            // menuItem6
            // 
            this.menuItem6.Index = 3;
            this.menuItem6.Text = "-";
            // 
            // menuInstanceOpen
            // 
            this.menuInstanceOpen.Index = 4;
            this.menuInstanceOpen.Text = "Open Instance";
            this.menuInstanceOpen.Click += new System.EventHandler(this.menuInstanceOpen_Click);
            // 
            // menuInstanceRestart
            // 
            this.menuInstanceRestart.Index = 5;
            this.menuInstanceRestart.Text = "Restart Process";
            this.menuInstanceRestart.Click += new System.EventHandler(this.menuInstanceRestart_Click);
            // 
            // menuItem4
            // 
            this.menuItem4.Index = 3;
            this.menuItem4.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuRotateCanvas,
            this.menuLayersManager});
            this.menuItem4.Text = "Canvas";
            // 
            // mnuRotateCanvas
            // 
            this.mnuRotateCanvas.Index = 0;
            this.mnuRotateCanvas.Text = "Rotate Canvas";
            this.mnuRotateCanvas.Click += new System.EventHandler(this.mnuRotateCanvas_Click);
            // 
            // menuLayersManager
            // 
            this.menuLayersManager.Index = 1;
            this.menuLayersManager.Text = "LayersManager";
            this.menuLayersManager.Click += new System.EventHandler(this.menuLayersManager_Click);
            // 
            // menuItem14
            // 
            this.menuItem14.Index = 4;
            this.menuItem14.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuManual,
            this.menuTutorial,
            this.menuItem17,
            this.mnuSendFeedBack,
            this.menuItem25,
            this.menuAboutModeler,
            this.menuItem2,
            this.menuItem15,
            this.menuAboutScenario,
            this.menuAboutProAndPro,
            this.menuItem18,
            this.menuWWWLinks});
            this.menuItem14.Text = "Help";
            // 
            // menuManual
            // 
            this.menuManual.Index = 0;
            this.menuManual.Text = "Table of contents";
            this.menuManual.Click += new System.EventHandler(this.menuManual_Click);
            // 
            // menuTutorial
            // 
            this.menuTutorial.Index = 1;
            this.menuTutorial.Text = "Tutorial";
            this.menuTutorial.Click += new System.EventHandler(this.menuTutorial_Click);
            // 
            // menuItem17
            // 
            this.menuItem17.Index = 2;
            this.menuItem17.Text = "-";
            // 
            // mnuSendFeedBack
            // 
            this.mnuSendFeedBack.Index = 3;
            this.mnuSendFeedBack.Text = "Send a FeedBack";
            this.mnuSendFeedBack.Click += new System.EventHandler(this.mnuSendFeedBack_Click);
            // 
            // menuItem25
            // 
            this.menuItem25.Index = 4;
            this.menuItem25.Text = "-";
            // 
            // menuAboutModeler
            // 
            this.menuAboutModeler.Index = 5;
            this.menuAboutModeler.Text = "About Scenario Modeler";
            this.menuAboutModeler.Click += new System.EventHandler(this.menuAboutModeler_Click);
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 6;
            this.menuItem2.Text = "FAQ on the Web";
            // 
            // menuItem15
            // 
            this.menuItem15.Index = 7;
            this.menuItem15.Text = "-";
            // 
            // menuAboutScenario
            // 
            this.menuAboutScenario.Index = 8;
            this.menuAboutScenario.Text = "About Scenario";
            this.menuAboutScenario.Click += new System.EventHandler(this.menuAboutScenario_Click);
            // 
            // menuAboutProAndPro
            // 
            this.menuAboutProAndPro.Index = 9;
            this.menuAboutProAndPro.Text = "About ProAndPro";
            // 
            // menuItem18
            // 
            this.menuItem18.Index = 10;
            this.menuItem18.Text = "-";
            // 
            // menuWWWLinks
            // 
            this.menuWWWLinks.Index = 11;
            this.menuWWWLinks.Text = "WWW Links";
            this.menuWWWLinks.Click += new System.EventHandler(this.menuWWWLinks_Click);
            // 
            // menuItem7
            // 
            this.menuItem7.Index = 5;
            this.menuItem7.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuStencilInfosName,
            this.mnuWindowSettings,
            this.menuItem10,
            this.menuItem23});
            this.menuItem7.Text = "SuperUser";
            // 
            // menuStencilInfosName
            // 
            this.menuStencilInfosName.Index = 0;
            this.menuStencilInfosName.Text = "Stencil Infos - Name";
            this.menuStencilInfosName.Click += new System.EventHandler(this.menuStencilInfosName_Click);
            // 
            // mnuWindowSettings
            // 
            this.mnuWindowSettings.Index = 1;
            this.mnuWindowSettings.Text = "Window Settings";
            this.mnuWindowSettings.Click += new System.EventHandler(this.mnuWindowSettings_Click);
            // 
            // menuItem10
            // 
            this.menuItem10.Index = 2;
            this.menuItem10.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem16,
            this.menuItem19,
            this.menuItem20});
            this.menuItem10.Text = "EasterEgg";
            // 
            // menuItem16
            // 
            this.menuItem16.Index = 0;
            this.menuItem16.Text = "Crea Shape da codice";
            this.menuItem16.Click += new System.EventHandler(this.menuItem16_Click);
            // 
            // menuItem19
            // 
            this.menuItem19.Index = 1;
            this.menuItem19.Text = "Crea Shape da codice per ogni Shape";
            this.menuItem19.Click += new System.EventHandler(this.menuItem19_Click);
            // 
            // menuItem20
            // 
            this.menuItem20.Index = 2;
            this.menuItem20.Text = "Invocare un comando Visio";
            this.menuItem20.Click += new System.EventHandler(this.menuItem20_Click);
            // 
            // menuItem23
            // 
            this.menuItem23.Index = 3;
            this.menuItem23.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuXSDEditor,
            this.menuXMLDB,
            this.menuXSDInference});
            this.menuItem23.Text = "XML";
            // 
            // mnuXSDEditor
            // 
            this.mnuXSDEditor.Index = 0;
            this.mnuXSDEditor.Text = "XSD Editor";
            this.mnuXSDEditor.Click += new System.EventHandler(this.mnuXSDEditor_Click);
            // 
            // menuXMLDB
            // 
            this.menuXMLDB.Index = 1;
            this.menuXMLDB.Text = "XML DB";
            this.menuXMLDB.Click += new System.EventHandler(this.menuXMLDB_Click);
            // 
            // menuXSDInference
            // 
            this.menuXSDInference.Index = 2;
            this.menuXSDInference.Text = "XSD Inference";
            this.menuXSDInference.Click += new System.EventHandler(this.menuXSDInference_Click);
            // 
            // shapeShortcutMenu
            // 
            this.shapeShortcutMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.shortcutMenu});
            // 
            // shortcutMenu
            // 
            this.shortcutMenu.Index = 0;
            this.shortcutMenu.Text = "Attachments";
            this.shortcutMenu.Click += new System.EventHandler(this.shortcutMenu_Click);
            // 
            // axDrawingControl1
            // 
            this.axDrawingControl1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.axDrawingControl1.Enabled = true;
            this.axDrawingControl1.Location = new System.Drawing.Point(0, 0);
            this.axDrawingControl1.Name = "axDrawingControl1";
            this.axDrawingControl1.OcxState = ((System.Windows.Forms.AxHost.State)(resources.GetObject("axDrawingControl1.OcxState")));
            this.axDrawingControl1.Size = new System.Drawing.Size(1112, 611);
            this.axDrawingControl1.TabIndex = 0;
            this.axDrawingControl1.MouseUpEvent += new AxMicrosoft.Office.Interop.VisOcx.EVisOcx_MouseUpEventHandler(this.axDrawingControl1_MouseUpEvent);
            this.axDrawingControl1.BeforeWindowClosed += new AxMicrosoft.Office.Interop.VisOcx.EVisOcx_BeforeWindowClosedEventHandler(this.axDrawingControl1_BeforeWindowClosed);
            this.axDrawingControl1.PageAdded += new AxMicrosoft.Office.Interop.VisOcx.EVisOcx_PageAddedEventHandler(this.axDrawingControl1_PageAdded);
            this.axDrawingControl1.BeforeShapeTextEdit += new AxMicrosoft.Office.Interop.VisOcx.EVisOcx_BeforeShapeTextEditEventHandler(this.axDrawingControl1_BeforeShapeTextEdit);
            this.axDrawingControl1.ShapeAdded += new AxMicrosoft.Office.Interop.VisOcx.EVisOcx_ShapeAddedEventHandler(this.axDrawingControl1_ShapeAdded);
            this.axDrawingControl1.BeforePageDelete += new AxMicrosoft.Office.Interop.VisOcx.EVisOcx_BeforePageDeleteEventHandler(this.axDrawingControl1_BeforePageDelete);
            // 
            // toolStrip1
            // 
            this.toolStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.toolStripButton1,
            this.toolStripButton3,
            this.toolStripButton2,
            this.toolStripSeparator1,
            this.toolStripButton4,
            this.toolStripPartecipants,
            this.toolStripButton5,
            this.toolStripButton6,
            this.toolStripSeparator2,
            this.toolStripButton7,
            this.toolStripSeparator3,
            this.toolStripButton8});
            this.toolStrip1.LayoutStyle = System.Windows.Forms.ToolStripLayoutStyle.HorizontalStackWithOverflow;
            this.toolStrip1.Location = new System.Drawing.Point(0, 0);
            this.toolStrip1.Name = "toolStrip1";
            this.toolStrip1.Size = new System.Drawing.Size(1112, 36);
            this.toolStrip1.TabIndex = 8;
            this.toolStrip1.Text = "toolStrip1";
            // 
            // toolStripButton1
            // 
            this.toolStripButton1.Image = global::Scenario.Modeler.Properties.Resources.toolStripButton1_Image;
            this.toolStripButton1.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton1.Name = "toolStripButton1";
            this.toolStripButton1.Size = new System.Drawing.Size(42, 33);
            this.toolStripButton1.Text = "Nuovo";
            this.toolStripButton1.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton1.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            // 
            // toolStripButton3
            // 
            this.toolStripButton3.Image = global::Scenario.Modeler.Properties.Resources.apriToolStripMenuItem_Image;
            this.toolStripButton3.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton3.Name = "toolStripButton3";
            this.toolStripButton3.Size = new System.Drawing.Size(30, 33);
            this.toolStripButton3.Text = "Apri";
            this.toolStripButton3.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton3.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripButton3.ToolTipText = "Apri";
            // 
            // toolStripButton2
            // 
            this.toolStripButton2.Image = global::Scenario.Modeler.Properties.Resources.toolStripButton2_Image;
            this.toolStripButton2.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton2.Name = "toolStripButton2";
            this.toolStripButton2.Size = new System.Drawing.Size(37, 33);
            this.toolStripButton2.Text = "Salva";
            this.toolStripButton2.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton2.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripButton2.ToolTipText = "Salva";
            // 
            // toolStripSeparator1
            // 
            this.toolStripSeparator1.Name = "toolStripSeparator1";
            this.toolStripSeparator1.Size = new System.Drawing.Size(6, 36);
            // 
            // toolStripButton4
            // 
            this.toolStripButton4.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton4.Name = "toolStripButton4";
            this.toolStripButton4.Size = new System.Drawing.Size(41, 33);
            this.toolStripButton4.Text = "Eventi";
            this.toolStripButton4.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton4.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripButton4.ToolTipText = "Salva";
            // 
            // toolStripPartecipants
            // 
            this.toolStripPartecipants.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripPartecipants.Name = "toolStripPartecipants";
            this.toolStripPartecipants.Size = new System.Drawing.Size(68, 33);
            this.toolStripPartecipants.Text = "Partecipanti";
            this.toolStripPartecipants.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripPartecipants.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripPartecipants.ToolTipText = "Salva";
            // 
            // toolStripButton5
            // 
            this.toolStripButton5.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton5.Name = "toolStripButton5";
            this.toolStripButton5.Size = new System.Drawing.Size(58, 33);
            this.toolStripButton5.Text = "DataType";
            this.toolStripButton5.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton5.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripButton5.ToolTipText = "Salva";
            // 
            // toolStripButton6
            // 
            this.toolStripButton6.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton6.Name = "toolStripButton6";
            this.toolStripButton6.Size = new System.Drawing.Size(63, 33);
            this.toolStripButton6.Text = "Application";
            this.toolStripButton6.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton6.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripButton6.ToolTipText = "Salva";
            // 
            // toolStripSeparator2
            // 
            this.toolStripSeparator2.Name = "toolStripSeparator2";
            this.toolStripSeparator2.Size = new System.Drawing.Size(6, 36);
            // 
            // toolStripButton7
            // 
            this.toolStripButton7.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton7.Name = "toolStripButton7";
            this.toolStripButton7.Size = new System.Drawing.Size(58, 33);
            this.toolStripButton7.Text = "Convalida";
            this.toolStripButton7.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton7.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripButton7.ToolTipText = "Salva";
            // 
            // toolStripSeparator3
            // 
            this.toolStripSeparator3.Name = "toolStripSeparator3";
            this.toolStripSeparator3.Size = new System.Drawing.Size(6, 36);
            // 
            // toolStripButton8
            // 
            this.toolStripButton8.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.creatToolStripMenuItem,
            this.eseguiToolStripMenuItem,
            this.apriToolStripMenuItem,
            this.restartToolStripMenuItem});
            this.toolStripButton8.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton8.Name = "toolStripButton8";
            this.toolStripButton8.Size = new System.Drawing.Size(113, 33);
            this.toolStripButton8.Text = "Workflow Instance";
            this.toolStripButton8.TextDirection = System.Windows.Forms.ToolStripTextDirection.Horizontal;
            this.toolStripButton8.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageAboveText;
            this.toolStripButton8.ToolTipText = "Salva";
            // 
            // creatToolStripMenuItem
            // 
            this.creatToolStripMenuItem.Name = "creatToolStripMenuItem";
            this.creatToolStripMenuItem.Size = new System.Drawing.Size(121, 22);
            this.creatToolStripMenuItem.Text = "Crea";
            // 
            // eseguiToolStripMenuItem
            // 
            this.eseguiToolStripMenuItem.Name = "eseguiToolStripMenuItem";
            this.eseguiToolStripMenuItem.Size = new System.Drawing.Size(121, 22);
            this.eseguiToolStripMenuItem.Text = "Esegui";
            // 
            // apriToolStripMenuItem
            // 
            this.apriToolStripMenuItem.Name = "apriToolStripMenuItem";
            this.apriToolStripMenuItem.Size = new System.Drawing.Size(121, 22);
            this.apriToolStripMenuItem.Text = "Apri";
            // 
            // restartToolStripMenuItem
            // 
            this.restartToolStripMenuItem.Name = "restartToolStripMenuItem";
            this.restartToolStripMenuItem.Size = new System.Drawing.Size(121, 22);
            this.restartToolStripMenuItem.Text = "Restart";
            // 
            // MainForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(1112, 611);
            this.Controls.Add(this.toolStrip1);
            this.Controls.Add(this.axDrawingControl1);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Menu = this.mainMenu1;
            this.Name = "MainForm";
            this.Text = "Scenario Modeler";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            ((System.ComponentModel.ISupportInitialize)(this.axDrawingControl1)).EndInit();
            this.toolStrip1.ResumeLayout(false);
            this.toolStrip1.PerformLayout();
            this.ResumeLayout(false);
            this.PerformLayout();

		}
		#endregion

		
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
            WorkflowEngine.Engine.Instance.StartUp(String.Empty, false);
            //Scenario.WorkflowEngine.Engine.DataTypesHelper.GetDataTypes();
            //WorkflowEngine.Engine.DataTypesHelper.GetDataTypes();
            
			/*
			SplashScreen.ShowSplashScreen(); 
			System.Windows.Forms.Application.DoEvents();
			SplashScreen.SetStatus("Loading module 1");
			System.Threading.Thread.Sleep(500);
			SplashScreen.SetStatus("Loading module 2");
			System.Threading.Thread.Sleep(300);
			SplashScreen.SetStatus("Loading module 3");
			System.Threading.Thread.Sleep(900);
			SplashScreen.SetStatus("Loading module 4");
			System.Threading.Thread.Sleep(100);
			SplashScreen.SetStatus("Loading module 5");
			System.Threading.Thread.Sleep(400);
			SplashScreen.SetStatus("Loading module 6");
			System.Threading.Thread.Sleep(50);
			SplashScreen.SetStatus("Loading module 7");
			System.Threading.Thread.Sleep(240);
			SplashScreen.SetStatus("Loading module 8");
			System.Threading.Thread.Sleep(900);
			SplashScreen.SetStatus("Loading module 9");
			System.Threading.Thread.Sleep(240);
			SplashScreen.SetStatus("Loading module 10");
			System.Threading.Thread.Sleep(90);
			SplashScreen.SetStatus("Loading module 11");
			System.Threading.Thread.Sleep(1000);
			SplashScreen.SetStatus("Loading module 12");
			System.Threading.Thread.Sleep(100);
			SplashScreen.SetStatus("Loading module 13");
			System.Threading.Thread.Sleep(500);
			*/
			Forms.Login login = new Forms.Login();
			System.Windows.Forms.Application.Run(login);

			System.Windows.Forms.Application.Run(new MainForm());

            //WorkflowEngine.Engine.Instance.ShutDown();
		}


		
        
		private void OpenDocument(string sSrc)
		{
			xmlRoles = new WorkflowParticipantsData.XMLRoles();
			xmlWorkflows = new WorkflowProcesses.XMLWorkflows();

			this.axDrawingControl1.Src = sSrc;

			try
			{
				auditDocument=this.axDrawingControl1.Document;
				pagesInDocument = auditDocument.Pages;

				string sSolutionXML = this.GetSolutionXML (auditDocument, "Participants");
				if ( sSolutionXML != string.Empty  )
				{
					xmlRoles.Parse ( sSolutionXML);
				}
				sSolutionXML = this.GetSolutionXML (auditDocument, "Workflows");
				if ( sSolutionXML != string.Empty )
				{
					xmlWorkflows.Parse ( sSolutionXML);
				}
				else
				{
					sSolutionXML = this.GetSolutionXML (auditDocument, "Workflow");
					WorkflowProcesses.XMLWorkflow xmlWorkflow = new WorkflowProcesses.XMLWorkflow();
					if ( sSolutionXML != string.Empty )
					{
						xmlWorkflow.Parse (sSolutionXML);

						if ( xmlWorkflow.Name == string.Empty )
						{
							xmlWorkflow.Name = "Pagina-1";
						}

						this.axDrawingControl1.Document.Application.ActivePage.Name = xmlWorkflow.Name;
					}
					else
					{
						xmlWorkflow.Name = this.axDrawingControl1.Document.Application.ActivePage.Name;
					}
					xmlWorkflows.Add (xmlWorkflow);
				}

				Utility.setUpVisioDrawing(this.axDrawingControl1);
			}
				//catch ( System.Runtime.InteropServices.COMException)
			catch ( System.Exception se)
			{
				MessageBox.Show ("Impossible to open document " + sSrc + "." +  se.Message);
				return;
			}
		}


		private WorkflowRelevantData.XMLAttributes RetrieveAttributes()
		{
			WorkflowRelevantData.XMLAttributes vAttributes = new WorkflowRelevantData.XMLAttributes();

			try
			{
				//fill attribute
				Visio.Shape shapeInPage;
				Visio.Page currentPage;

				for (int pageIndex = 1; pageIndex <= pagesInDocument.Count; pageIndex++) 
				{
					System.Console.WriteLine("carica albero");
					currentPage = pagesInDocument[pageIndex];
					
					for ( int iShape = 1; iShape<=currentPage.Shapes.Count; iShape++)
					{
						shapeInPage = currentPage.Shapes[iShape];

						String sShapeId = Utility.GetMasterNameU(shapeInPage);

						switch (sShapeId)
						{							
							case Attribute_MASTER:
								if ( ( shapeInPage.Data1 != null ) && (shapeInPage.Data1 != string.Empty ))
								{
									WorkflowRelevantData.XMLAttributes vAttributes2Add = new WorkflowRelevantData.XMLAttributes();
									vAttributes2Add.Parse(shapeInPage.Data1);

									for ( int i=0; i<vAttributes2Add.Count; i++)
									{
										WorkflowRelevantData.XMLAttribute attribute = (WorkflowRelevantData.XMLAttribute)vAttributes2Add[i];

										vAttributes.Add (attribute);
									}
								}
								break;
						}
					}
				}
			}
			catch(Exception err)
			{
				System.Console.WriteLine(err.Message);
				throw(err);
			}

			return vAttributes;
		}

		

		private bool OpenShapeForm(Visio.Shape shape, bool bFromStencil)
		{
			bool bRet = true;

			String sShapeId = Utility.GetMasterNameU(shape);

			switch (sShapeId)
			{
				case Start_MASTER:
					WorkflowProcesses.XMLActivity activityStart = new WorkflowProcesses.XMLActivity();
					activityStart.SetToStart ();
					shape.Data1 = activityStart.GetXML();
					//shape.Text = "Start";
					break;

				case End_MASTER:
					WorkflowProcesses.XMLActivity activityEnd = new WorkflowProcesses.XMLActivity();
					activityEnd.SetToEnd();
					shape.Data1 = activityEnd.GetXML();
					//shape.Text = "End";
					break;

				case Gateway_MASTER:
				case GatewayOR_MASTER:
				case GatewayAND_MASTER:
				case GatewayXOR_MASTER:
				case GatewayEvent_MASTER:
				case GatewayComplex:
				case Activity_MASTER:
				case COLLAPSED_SUBPROCESS_MASTER:

					FormActivity oActivity=new FormActivity(shape, ref xmlRoles);
					oActivity.XMLAttributes = this.RetrieveAttributes();
					//oActivity.Show();
					oActivity.ShowDialog(this);

					bRet = (shape.Data1 != "");
					break;

				case Scenario.Modeler.Forms.BPMN.BPMNUtils.Flow_MASTER:

					if ( !bFromStencil ) 
					{
						FormTransition oTransition=new FormTransition(shape);
						oTransition.ShowDialog(this);
						bRet = (shape.Data1 != "");
					}
					else
						bRet= true;

					break;

                case Scenario.Modeler.Forms.BPMN.BPMNUtils.MessageFlow_MASTER:

                    if (!bFromStencil)
                    {
                        Scenario.Modeler.Forms.BPMN.MessageFlow oMessage =
                                new Scenario.Modeler.Forms.BPMN.MessageFlow(shape);
                        oMessage.ShowDialog(this);
                        bRet = (shape.Data1 != "");
                    }
                    else
                        bRet = true;

                    break;
							
				case Attribute_MASTER:
					FormAttributes oAttribute=new FormAttributes(shape);
					oAttribute.ShowDialog(this);
					bRet = (shape.Data1 != "");
					
					break;

				case "Blank form":
					Visio.Page page = this.axDrawingControl1.Document.Pages.Add();
					break;

				case "Text box (single-line)":
				case "Text box (multi-line)":
				case "Combo box":
				/*case "Grid column":
					Scenario.Modeler.PCM.EntityAttribute form = new Scenario.Modeler.PCM.EntityAttribute(shape.Data1, shape.Data2, shape.Data3);
					form.ShowDialog (this);

					if ( form.NotCancel ) 
					{
						shape.Data1 = form.xmlEntityAttribute.GetXML();
						shape.Data2 = form.RelationShip.GetXML();
						shape.Data3 = form.RulesFamily.GetXML();

						shape.Text = form.xmlEntityAttribute.EntityRef + "." + form.xmlEntityAttribute.Name;
					}

					break;
                    */
						
				default:

					break;
			}

			return bRet;
		}

		

		#region Menu Event

		private void menuProcessProperties_Click(object sender, System.EventArgs e)
		{
			Visio.Page currentPage = this.axDrawingControl1.Document.Application.ActivePage;
			Scenario.Modeler.Forms.ProcessProperties properties = new Scenario.Modeler.Forms.ProcessProperties(currentPage.Name, ref this.xmlWorkflows);
			properties.ShowDialog (this);

			if ( properties.xmlWorkflow != null )
			{
				currentPage.Name = properties.xmlWorkflow.Name;
			}
		}

		private void mnuRotateCanvas_Click(object sender, System.EventArgs e)
		{
			if ( this.mnuRotateCanvas.Checked )
			{
				this.axDrawingControl1.Document.PrintLandscape = false;
			}
			else
			{
				this.axDrawingControl1.Document.PrintLandscape = true;
			}

			Utility.setUpVisioDrawing(this.axDrawingControl1);
		}

		private void mnuOpenInstance_Click(object sender, System.EventArgs e)
		{
			
		}

		private void mnuCreateTemplate_Click(object sender, System.EventArgs e)
		{
			try
			{
                /*
				WAPI.Response response = new WAPI.Response();

				SaveFileDialog saveFileDialog1 = new SaveFileDialog();
 
				string sWorkflowProcess = WAPI.Session.GetDirectory (WAPI.Session.WORKFLOWPROCESS_DIRECTORY, ref response);

				if ( response.iCode == 0 )
				{
					//saveFileDialog1.InitialDirectory = sWorkflowProcess;

					// Default to the My Documents folder.
					this.folderBrowserDialog1.SelectedPath= sWorkflowProcess;//Environment.SpecialFolder.Personal;;
				}
                */
				//saveFileDialog1.Filter = "Scenario files (*.xml)|*.xml"  ;

				this.folderBrowserDialog1.Description = 
					"Select the directory that you want to use as the default.";

				// Do not allow the user to create new files via the FolderBrowserDialog.
				this.folderBrowserDialog1.ShowNewFolderButton = true;

				
				/*
				saveFileDialog1.FilterIndex = 1 ;
				saveFileDialog1.RestoreDirectory = true ;
				*/
 
				/*
				if(saveFileDialog1.ShowDialog() == DialogResult.OK)
				{
					
					// Code to write the stream goes here.
					StreamWriter sw = File.CreateText(saveFileDialog1.FileName);
					sw.Write(xmlWorkflow.GetXML());
					sw.Close();
				}
				*/
				// Show the FolderBrowserDialog.
				DialogResult result = this.folderBrowserDialog1.ShowDialog();
				if( result == DialogResult.OK )
				{
					string sWarnings = this.BuildModel();
					if ( sWarnings != string.Empty )
					{
						if ( MessageBox.Show (this, sWarnings, "Warnings", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No )
						{
							return;
						}
					}

					string sFolderName = this.folderBrowserDialog1.SelectedPath;

					WorkflowProcesses.XMLWorkflow xmlWorkflow = null;

					for ( int i=0; i<this.xmlWorkflows.Count;i++)
					{
						xmlWorkflow = (WorkflowProcesses.XMLWorkflow)this.xmlWorkflows[i];
						string sWfName = sFolderName + "\\" + xmlWorkflow.Name + ".xml";

						StreamWriter sw = File.CreateText(sWfName);
						sw.Write(xmlWorkflow.GetXML());
						sw.Close();
					}
				}
			}
			catch (Exception err)
			{
				MessageBox.Show ("not build file: " + err.Message);
			}
		}

		private void mnuStartTemplate_Click(object sender, System.EventArgs e)
		{
			FormStartProcess form = new FormStartProcess();
			form.ShowDialog (this);
		}

		private void mnuPrintPreview_Click(object sender, System.EventArgs e)
		{
			PrintPreview printPreviewForm = new PrintPreview();
			printPreviewForm.PreviewDrawing(axDrawingControl1.Document);
			printPreviewForm.ShowDialog();
		}

		private void menuSaveasWebPage_Click(object sender, System.EventArgs e)
		{
			SaveAsWebApi.SaveDocAsWebPage (axDrawingControl1.Document, -1,-1,"c:\\temp\\prova",
				SaveAsWebApi.ShowNavigationBar + SaveAsWebApi.ShowPanAndZoom + SaveAsWebApi.ShowPropertiesWindow+SaveAsWebApi.ShowSearchTool);
		}

		private void menuStencilInfosName_Click(object sender, System.EventArgs e)
		{
			AnchorBarForm bar = new AnchorBarForm();
			bar.ParentVisioApplication = this.axDrawingControl1.Window.Application;
			
			/*
			bar.ParentVisioApplication = this.axDrawingControl1.Window.Application;
			bar.Show();
			*/
			AnchorBars anchor= new AnchorBars();

			anchor.Anchor (this.axDrawingControl1.Window.Application, bar);
		}

		private void mnuWindowSettings_Click(object sender, System.EventArgs e)
		{
			VisioWindows form = new VisioWindows();
			form.DrawingControl = this.axDrawingControl1;
			form.Show();
		}

		private void menuItem16_Click(object sender, System.EventArgs e)
		{
			Visio.Page page = pagesInDocument[1];
			Scenario.Modeler.Utility.CreateTextOnlyShape(page,"Warning", "prova");
		}

		private void menuItem19_Click(object sender, System.EventArgs e)
		{
			Visio.Page currentPage = this.axDrawingControl1.Document.Application.ActivePage;
			Visio.Shape shapeInPage;

			Visio.Layer layer = currentPage.Layers.Add ("Execution1");
 
			for ( int iShape=1; iShape<=currentPage.Shapes.Count; iShape++)
			{
				shapeInPage = currentPage.Shapes[iShape];

				System.Console.WriteLine(shapeInPage.NameU);
				String sShapeId = Utility.GetMasterNameU(shapeInPage);

				switch (sShapeId)
				{
					case Gateway_MASTER:
					case Activity_MASTER:
					case COLLAPSED_SUBPROCESS_MASTER:
					case Flow_MASTER:
					case Attribute_MASTER:
					case End_MASTER:

						double dblTop;
						double dblBottom;
						double dblLeft;
						double dblRight;

						shapeInPage.BoundingBox ((short)Visio.VisBoundingBoxArgs.visBBoxExtents, out dblLeft, out dblBottom, out dblRight, out dblTop );
						Visio.Shape shape= currentPage.DrawRectangle (dblLeft, dblBottom, dblRight, dblTop-1);

						shape.Name = "Prova";
						shape.LineStyle = "Normal";
						shape.FillStyle = "Normal";
						shape.Text = "Prova" + shapeInPage.Text;

						/* Add a shape to the layer. This group's component shapes are
						' added to the layer. If preserveMembers is zero, the component 
						' shapes are reassigned to this layer and lose their current
						' layer assignments.
						*/
						layer.Add(shape, 0);
						break;
				}
			}
		}

		private void menuLayersManager_Click(object sender, System.EventArgs e)
		{
			LayersManager bar = new LayersManager();
			bar.Pages = this.axDrawingControl1.Document.Pages;

			AnchorBars anchor= new AnchorBars();
			anchor.Anchor (this.axDrawingControl1.Window.Application, bar);
		}

		private void menuItem20_Click(object sender, System.EventArgs e)
		{
			OleCommandTarget.IOleCommandTarget commandTarget = 
				(OleCommandTarget.IOleCommandTarget)this.axDrawingControl1.GetOcx();

			try
			{
				Guid CLSID_Application = new Guid("{0x00021A20, 0x0000, 0x0000, {0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46}}");
				commandTarget.Exec(ref CLSID_Application, (uint)Visio.VisUICmds.visCmdFileNew, 0, null, null);
			}
			catch(System.Runtime.InteropServices.COMException ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}
		private void menuCheck_Click(object sender, System.EventArgs e)
		{
			System.Xml.XmlDocument document = new System.Xml.XmlDocument();

			try
			{
				WAPI.Response response = new WAPI.Response();

                string sChecks = WorkflowEngine.Engine.FolderHelper.GetFolder(WAPI.FolderHelper.FolderType.Execution);

				if ( response.iCode == 0 )
				{
					string sDirectory = sChecks +"\\bin\\Checks";
					string sFileManager = sDirectory + "\\checkslist.xml";

					document.Load (sFileManager);
					System.Xml.XmlNode root = document.FirstChild;

					this.BuildModel();
					for ( int i=0; i<root.ChildNodes.Count;i++)
					{
						Scenario.Modeler.Forms.XSLTCheck formCheck = new Scenario.Modeler.Forms.XSLTCheck();
						formCheck.XSLT = sDirectory + "\\"  + root.ChildNodes[i].Attributes["File"].Value + ".xslt";
				
						WorkflowProcesses.XMLWorkflow xmlWorkflow = null;

						for ( int j=0; j<this.xmlWorkflows.Count;j++)
						{
							xmlWorkflow = (WorkflowProcesses.XMLWorkflow )this.xmlWorkflows[j];
	
							formCheck.XML = xmlWorkflow.GetXML();
	
							formCheck.ShowDialog (this);
						}

					}
				}
			}
			catch (System.Xml.XmlException err)
			{
				System.Console.WriteLine ("Error reading checklist.xml [ {0} ]", err.Message);
			}
		}

		private void menuTutorial_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuManual_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuAboutScenario_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuAboutModeler_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuWWWLinks_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuCreateModel_Click(object sender, System.EventArgs e)
		{
			
		}

		private void menuResources_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuNew_Click(object sender, System.EventArgs e)
		{
			
		}

		private void mnuOpen_Click(object sender, System.EventArgs e)
		{
			OpenFileDialog openFileDialog1 = new OpenFileDialog();

			openFileDialog1.Filter = "vsd files (*.vsd)|*.vsd";
			openFileDialog1.FilterIndex = 1 ;
			openFileDialog1.RestoreDirectory = true ;

			if(openFileDialog1.ShowDialog() == DialogResult.OK)
			{
				// Open the new document.
				string FileName;
				FileName=openFileDialog1.FileName;

				axDrawingControl1.Document.Saved = true;
				
				this.OpenDocument (openFileDialog1.FileName);
			}
		}

		private void menuSave_Click(object sender, System.EventArgs e)
		{
			this.AddSolutionXML (axDrawingControl1.Document, "Participants",this.xmlRoles.GetXML()); 
			this.AddSolutionXML (axDrawingControl1.Document, "Workflows",this.xmlWorkflows.GetXML()); 

			axDrawingControl1.Document.SaveAs(axDrawingControl1.Src );
		}

		private void mnuSaveAs_Click(object sender, System.EventArgs e)
		{
			SaveFileDialog saveFileDialog1 = new SaveFileDialog();
 
			saveFileDialog1.Filter = "Scenario File (*.vsd)|*.vsd"  ;
			saveFileDialog1.FilterIndex = 1 ;
			saveFileDialog1.RestoreDirectory = true ;
 
			if(saveFileDialog1.ShowDialog() == DialogResult.OK)
			{
				this.AddSolutionXML (axDrawingControl1.Document, "Participants",this.xmlRoles.GetXML()); 
				this.AddSolutionXML (axDrawingControl1.Document, "Workflow",this.xmlWorkflows.GetXML()); 

				axDrawingControl1.Document.SaveAs(saveFileDialog1.FileName);
				axDrawingControl1.Src = saveFileDialog1.FileName;

                this.OpenDocument(saveFileDialog1.FileName);
			}
		}

		

		private void menuRecentFiles_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuExit_Click(object sender, System.EventArgs e)
		{
			this.Close();
			System.Windows.Forms.Application.Exit();
		}

		private void menuDataTypes_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private void menuApplications_Click(object sender, System.EventArgs e)
		{
			Applications applications = new Applications();
			applications.ShowDialog (this);
		}

		private void menuPartecipants_Click(object sender, System.EventArgs e)
		{
			Participants participants = new Participants(xmlRoles);
			participants.ShowDialog (this);

			if ( participants.xmlRoles != null )
			{
				this.xmlRoles = participants.xmlRoles;
			}
		}

		private void menuManageCheck_Click(object sender, System.EventArgs e)
		{
			Scenario.Modeler.Forms.XSLTFilesManager manager = new Scenario.Modeler.Forms.XSLTFilesManager();
			manager.MainForm = this;
			manager.ShowDialog (this);
		}

		#endregion

		private void AddSolutionXML(Visio.Document document, string sKey, string sXML)
		{
			string sSolution = "<SolutionXML Name='";
			sSolution+=sKey;
			sSolution+="'>";
			sSolution+=sXML;
			sSolution+="</SolutionXML>";

			document.set_SolutionXMLElement(sKey, sSolution);
		}

		private string GetSolutionXML(Visio.Document document, string sKey)
		{
			string sSolution;
			string sValue = string.Empty;

			try 
			{
				if (document.get_SolutionXMLElementExists(sKey)) 
				{

					sSolution = document.get_SolutionXMLElement(sKey);

					System.Xml.XmlDocument solutionXml;
					System.Xml.XmlElement rootElement;
					
					try 
					{
						solutionXml = new System.Xml.XmlDocument();
						solutionXml.LoadXml(sSolution);
			
						// Retrieve the XML data stored with the SolutionXML
						// element.
						rootElement = solutionXml.DocumentElement;
						sValue = rootElement.InnerXml;
					}
					catch (Exception err) 
					{
						System.Diagnostics.Debug.WriteLine("Exception: " + 	err.Message);				
					}
				}
			}

			catch (Exception err) 
			{
				System.Diagnostics.Debug.WriteLine(err.Message);
			}

			return sValue;
		}

		
		public string BuildModel()
		{
			Visio.Shape shapeInPage;
			Visio.Page currentPage;

			string sWarningMessages = string.Empty;

			for (int pageIndex = 1; pageIndex <= this.pagesInDocument.Count; pageIndex++) 
			{
				currentPage = pagesInDocument[pageIndex];
				bool bStartAlreadyAdded = false;
				bool bEndAlreadyAdded = false;

				WorkflowProcesses.XMLWorkflow xmlWorkflow = null;
				
				for ( int i=0; i<this.xmlWorkflows.Count;i++)
				{
					if ( ((WorkflowProcesses.XMLWorkflow)this.xmlWorkflows[i]).Name == currentPage.Name)  
					{
						xmlWorkflow = (WorkflowProcesses.XMLWorkflow)this.xmlWorkflows[i];
						break;
					}
				}

				if ( xmlWorkflow == null ) 
				{
					sWarningMessages+=currentPage.Name + " not created.\n";
					continue;
				}

				xmlWorkflow.XMLActivities = new WorkflowProcesses.XMLActivities();
				xmlWorkflow.XMLAttributes = new WorkflowRelevantData.XMLAttributes();
				xmlWorkflow.XMLTransitions = new WorkflowProcesses.XMLTransitions();
                xmlWorkflow.XMLMessages = new WorkflowProcesses.XMLMessages();

				for ( int iShape=1; iShape<=currentPage.Shapes.Count; iShape++)
				{
					shapeInPage = currentPage.Shapes[iShape];

					System.Console.WriteLine(shapeInPage.NameU);
					String sShapeId = Utility.GetMasterNameU(shapeInPage);
					switch (sShapeId)
					{
                        case Scenario.Modeler.Forms.BPMN.BPMNUtils.MessageFlow_MASTER:
                            try
                            {
                                WorkflowProcesses.XMLMessage oMessage = new WorkflowProcesses.XMLMessage();
                                if (shapeInPage.Data1 != "")
                                {
                                    oMessage.Parse(shapeInPage.Data1);
                                }
                                xmlWorkflow.XMLMessages.Add(oMessage);

                                Visio.Connects o_Connects = shapeInPage.Connects;

                                foreach (Visio.Connect o_Connect in o_Connects)
                                {
                                    Visio.Shape oShape = o_Connect.ToSheet;
                                    string sShapeIdConnect = Utility.GetMasterNameU(oShape);

                                    if ((sShapeIdConnect == Activity_MASTER) ||
                                        (sShapeIdConnect == Gateway_MASTER) ||
                                        (sShapeIdConnect == COLLAPSED_SUBPROCESS_MASTER))
                                    {
                                        WorkflowProcesses.XMLActivity activity2Com = new WorkflowProcesses.XMLActivity();
                                        activity2Com.Parse(oShape.Data1);
                                        int FlowDirection = o_Connect.FromPart;

                                        switch (FlowDirection)
                                        {
                                            case 9: // visBegin=9 Connection is from the begin point of a 1-D shape.
                                                // Set new transaction.from
                                                oMessage.From = activity2Com.Name;
                                                break;
                                            case 12: //visEnd 12 Connection is from the end of a 1-D shape.
                                                // Set new transaction.to
                                                oMessage.To = activity2Com.Name;
                                                break;
                                        }
                                    }
                                    else if (sShapeIdConnect == Start_MASTER)
                                    {

                                        int FlowDirection = o_Connect.FromPart;

                                        switch (FlowDirection)
                                        {
                                            case 9: // visBegin=9 Connection is from the begin point of a 1-D shape.
                                                // Set new transaction.from
                                                oMessage.From = "Start";
                                                break;
                                            case 12: //visEnd 12 Connection is from the end of a 1-D shape.
                                                // Set new transaction.to
                                                //TODO: Warning
                                                break;
                                        }
                                    }
                                    else if (sShapeIdConnect == End_MASTER)
                                    {

                                        int FlowDirection = o_Connect.FromPart;

                                        switch (FlowDirection)
                                        {
                                            case 9: // visBegin=9 Connection is from the begin point of a 1-D shape.
                                                // Set new transaction.from
                                                //TODO: Error
                                                break;
                                            case 12: //visEnd 12 Connection is from the end of a 1-D shape.
                                                // Set new transaction.to
                                                //TODO: Warning
                                                oMessage.To = "End";
                                                break;
                                        }
                                    }
                                    else if (sShapeIdConnect == Scenario.Modeler.Forms.BPMN.BPMNUtils.POOL_LANE_MASTER)
                                    {
                                        //find pool name
                                        for (int i = 1; i <= oShape.Shapes.Count; i++)
                                        {
                                            string sShapeIdConnectchild = oShape.Shapes[i].Text;
                                            if ( sShapeIdConnectchild.CompareTo (string.Empty ) !=  0) 
                                            {
                                                int FlowDirection = o_Connect.FromPart;   
                                                switch (FlowDirection)
                                                {
                                                    case 9: // visBegin=9 Connection is from the begin point of a 1-D shape.
                                                        // Set new transaction.from
                                                        oMessage.From = sShapeIdConnectchild;
                                                        break;
                                                    case 12: //visEnd 12 Connection is from the end of a 1-D shape.
                                                        // Set new transaction.to
                                                        oMessage.To = sShapeIdConnectchild;
                                                        break;
                                                }
                                                break;
                                            }
                                        }
                                    }

                                }
                            }
                            catch (System.Exception)
                            {
                                System.Console.WriteLine(shapeInPage.Name);
                            }
                            break;
                            
 

						case Flow_MASTER:
							try
							{
								WorkflowProcesses.XMLTransition oTransition=new WorkflowProcesses.XMLTransition();
								if ( shapeInPage.Data1 != "" )
								{
									oTransition.Parse(shapeInPage.Data1);
								}
								xmlWorkflow.XMLTransitions.Add(oTransition);

								Visio.Connects o_Connects=shapeInPage.Connects;
				
								foreach (Visio.Connect o_Connect in o_Connects)
								{
									Visio.Shape oShape=o_Connect.ToSheet;
									string sShapeIdConnect = Utility.GetMasterNameU(oShape);

									if ( (sShapeIdConnect==Activity_MASTER) || 
										(sShapeIdConnect==Gateway_MASTER) || 
										(sShapeIdConnect==COLLAPSED_SUBPROCESS_MASTER) )
									{
										WorkflowProcesses.XMLActivity activity2Com = new WorkflowProcesses.XMLActivity();
										activity2Com.Parse ( oShape.Data1);
										int FlowDirection=o_Connect.FromPart;
								
										switch (FlowDirection)
										{
											case 9: // visBegin=9 Connection is from the begin point of a 1-D shape.
												// Set new transaction.from
												oTransition.From=activity2Com.Name;
												break;
											case 12: //visEnd 12 Connection is from the end of a 1-D shape.
												// Set new transaction.to
												oTransition.To=activity2Com.Name;
												break;
										}
									}
									else if (sShapeIdConnect==Start_MASTER)
									{

										int FlowDirection=o_Connect.FromPart;
								
										switch (FlowDirection)
										{
											case 9: // visBegin=9 Connection is from the begin point of a 1-D shape.
												// Set new transaction.from
												oTransition.From="Start";
												break;
											case 12: //visEnd 12 Connection is from the end of a 1-D shape.
												// Set new transaction.to
												//TODO: Warning
												break;
										}
									}
									else if (sShapeIdConnect==End_MASTER)
									{

										int FlowDirection=o_Connect.FromPart;
								
										switch (FlowDirection)
										{
											case 9: // visBegin=9 Connection is from the begin point of a 1-D shape.
												// Set new transaction.from
												//TODO: Error
												break;
											case 12: //visEnd 12 Connection is from the end of a 1-D shape.
												// Set new transaction.to
												//TODO: Warning
												oTransition.To="End";
												break;
										}
									}

								}
							}
							catch ( System.Exception )
							{
								System.Console.WriteLine(shapeInPage.Name);
							}
							break;

						case Attribute_MASTER:
							if ( shapeInPage.Data1 != "" ) 
							{
								WorkflowRelevantData.XMLAttributes attributes = new WorkflowRelevantData.XMLAttributes();
								attributes.Parse (shapeInPage.Data1);
								xmlWorkflow.AddAttributes (attributes);
							}
							break;

						case Start_MASTER:
							if ( bStartAlreadyAdded )
							{
								//TODO: Error
							}
							else
							{
								WorkflowProcesses.XMLActivity oActivityStart = xmlWorkflow.AddStartActivity();
								bStartAlreadyAdded = true;
							}
							break;


						case End_MASTER:
							if ( !bEndAlreadyAdded )
							{
								WorkflowProcesses.XMLActivity oActivityEnd= xmlWorkflow.AddEndActivity();
								bEndAlreadyAdded = true;
							}
							break;

						case Gateway_MASTER:
						case Activity_MASTER:
						case COLLAPSED_SUBPROCESS_MASTER:
						default:
							try
							{
								WorkflowProcesses.XMLActivity oActivity=new WorkflowProcesses.XMLActivity();
								oActivity.Parse(shapeInPage.Data1);

								xmlWorkflow.XMLActivities.Add(oActivity);
							}
							catch ( System.Exception)
							{
								System.Console.WriteLine(shapeInPage.Name);
							}
							break;

					}
				}
			}

			return sWarningMessages;
		}

		private void mnuXSDEditor_Click(object sender, System.EventArgs e)
		{
			xmlSchemaEditor.SchemaEditor form = new xmlSchemaEditor.SchemaEditor();
			form.Show();
		}

		private void menuXMLDB_Click(object sender, System.EventArgs e)
		{
			xmlDbEditor.Form1 form = new xmlDbEditor.Form1();
			form.Show();
		}

		private void menuXSDInference_Click(object sender, System.EventArgs e)
		{
			Scenario.Modeler.XSD.XSDInference form = new Scenario.Modeler.XSD.XSDInference();
			form.Show();
		}

		private void menuNewPCM_Click(object sender, System.EventArgs e)
		{
			Visio.Page page = this.axDrawingControl1.Document.Pages.Add();
		}

		private void menuNewBP_Click(object sender, System.EventArgs e)
		{
			Visio.Page page = this.axDrawingControl1.Document.Pages.Add();
			//Scenario.Modeler.Utility.CreateTextOnlyShape(page,"Warning", "Pagina non gestita");
		}

		private void mnuSendFeedBack_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show ("TODO");
		}

		private string GetPageName(string sWorkflowName)
		{
			WorkflowProcesses.XMLWorkflow xmlWorkflow = null;
			string sName = sWorkflowName;
			for (int i=1;;i++)
			{
				if ( i > 1 )
				{
					sName = sWorkflowName + "_" + i;
				}
				xmlWorkflow = this.xmlWorkflows.GetWorkflowByName (sName);

				if ( xmlWorkflow == null )
					return sName;
			}
		}

		private void LoadWorkflowInCanvas(WorkflowProcesses.XMLWorkflow xmlWorkflow)
		{
			bNotApplyAddEvent = true;
			Visio.Page page = this.axDrawingControl1.Document.Pages.Add();
			bNotApplyAddEvent = false;

			page.Name = this.GetPageName (xmlWorkflow.Name);
			this.xmlWorkflows.Add (xmlWorkflow);
			

			//ciclo sulle attivit.
			WorkflowProcesses.XMLActivity xmlActivity = null;
			Visio.Shape shape = null;

			double x=0.5;
			double y=7;
			double deltax=1;
			double deltay=0.5;
			for ( int i=0; i<xmlWorkflow.XMLActivities.Count;i++)
			{
				xmlActivity = (WorkflowProcesses.XMLActivity)xmlWorkflow.XMLActivities[i];

				shape = Utility.CreateTextOnlyShape (page, Activity_MASTER, xmlActivity.Name,x, y, deltax, deltay);
				shape.Data1 = xmlActivity.GetXML();

				/*
					 * Open Visio
					 * Open blanknew.vsd
					 * Define a new style in Format/DefineStyle
					 * 	shape.Style = "ActivityRunning";
					*/

				x=x+deltax+deltax;

				if ( x>10)
				{
					x = 7;
					deltax = -1;

					y = y-1;
				}

				if ( x<1)
				{
					x=0.5;
					y=y-1;
					deltax=1;
				}
			}


			//ciclo sulle transizioni.
			//ciclo sugli attributi.
			shape = Utility.CreateTextOnlyShape (page, Attribute_MASTER, "Attributes",1, 10, 1, 1);
			shape.Data1 = xmlWorkflow.XMLAttributes.GetXML();
		}

		private void menuInstanceOpen_Click(object sender, System.EventArgs e)
		{
			Forms.ProcessesList form = new Forms.ProcessesList ();
			form.ShowDialog ();

			if ( form.XMLWorkflow != null )
			{
				this.LoadWorkflowInCanvas (form.XMLWorkflow);
			}
		}

		private void menuInstanceRestart_Click(object sender, System.EventArgs e)
		{
			Visio.Page currentPage = this.axDrawingControl1.Document.Application.ActivePage;
			Scenario.Modeler.Forms.ProcessInstanceManager manager = new Scenario.Modeler.Forms.ProcessInstanceManager(currentPage.Name, ref this.xmlWorkflows);
			manager.ShowDialog (this);


			if ( manager.xmlWorkflow != null )
			{
				this.LoadWorkflowInCanvas (manager.xmlWorkflow);
			}
		}

        private void axDrawingControl1_BeforePageDelete(object sender, AxMicrosoft.Office.Interop.VisOcx.EVisOcx_BeforePageDeleteEvent e)
        {
            this.xmlWorkflows.RemoveProcess(e.page.Name);
        }

        private void axDrawingControl1_PageAdded(object sender, AxMicrosoft.Office.Interop.VisOcx.EVisOcx_PageAddedEvent e)
        {
            if (bNotApplyAddEvent == false)
            {
                WorkflowProcesses.XMLWorkflow xmlWorkflow = new WorkflowProcesses.XMLWorkflow();
                xmlWorkflow.Name = e.page.Name;
                this.xmlWorkflows.Add(xmlWorkflow);
            }
        }

        private void axDrawingControl1_ShapeAdded(object sender, AxMicrosoft.Office.Interop.VisOcx.EVisOcx_ShapeAddedEvent e)
        {
            try
            {
                this.OpenShapeForm(e.shape, true);
            }
            catch (Exception err)
            {
                System.Diagnostics.Debug.WriteLine(err.Message);
            }
        }

        private void axDrawingControl1_BeforeWindowClosed(object sender, AxMicrosoft.Office.Interop.VisOcx.EVisOcx_BeforeWindowClosedEvent e)
        {
            //Questo evento viene lanciato anche sul SaveAs per cui  indispensabile discriminare se si sta chiudendo il documento
            //oppure se si sta facendo semplicemente saveAs
            pagesInDocument = null;
            auditDocument = null;
            //auditDocument = this.axDrawingControl1.Document;
            //pagesInDocument = auditDocument.Pages;
            
        }

        private void axDrawingControl1_BeforeShapeTextEdit(object sender, AxMicrosoft.Office.Interop.VisOcx.EVisOcx_BeforeShapeTextEditEvent e)
        {
            this.OpenShapeForm(e.shape, false);
        }

        private void axDrawingControl1_MouseUpEvent(object sender, AxMicrosoft.Office.Interop.VisOcx.EVisOcx_MouseUpEvent e)
        {
            if ((e.button == (int)Visio.VisKeyButtonFlags.visMouseRight) &&
                ((e.keyButtonState & (int)Visio.VisKeyButtonFlags.visKeyControl) == 0))
            {

                // Look for a shape on the drawing page at the click location.
                Microsoft.Office.Interop.Visio.Shape clickedShape = Utility.GetClickedShape(axDrawingControl1,
                    e.x, e.y);

                if ((clickedShape != null) &&
                    ((clickedShape.Master.NameU == Start_MASTER)
                    || (clickedShape.Master.NameU == End_MASTER)
                    || (clickedShape.Master.NameU == Intermediate_MASTER)
                    || (clickedShape.Master.NameU == Activity_MASTER)
                    || (clickedShape.Master.NameU == Gateway_MASTER)
                    || (clickedShape.Master.NameU == Flow_MASTER)
                    || (clickedShape.Master.NameU == Attribute_MASTER)
                    ))
                {

                    // Cancel the default processing of this event since
                    // it will be handled in this method.
                    e.cancelDefault = true;

                    // Show the shortcut menu at the click location. The
                    // coordinates need to be converted from the Visio page
                    // units that are in the event data to Windows pixel
                    // coordinates.

                    shapeShortcutMenu.Tag = clickedShape;
                    shapeShortcutMenu.Show(this,
                        Utility.MapVisioToWindows(axDrawingControl1,
                        e.x, e.y));
                }
            }
        }

        private void shortcutMenu_Click(object sender, EventArgs e)
        {
            MenuItem mnuItem = (MenuItem)sender;
            ContextMenu menu = (ContextMenu)mnuItem.Parent;
            Microsoft.Office.Interop.Visio.Shape shape = (Microsoft.Office.Interop.Visio.Shape)menu.Tag;
            MessageBox.Show("Hai selezionato" + shape.Text);
        }

        private void menuConfiguration_Click(object sender, EventArgs e)
        {

        }











	}
}
