#define MAKE_NEW_PCM_ATTRIBUTES

using System;
using System.Data;
using System.Xml;
using System.Collections;
using System.Drawing;
using Scenario.Modules.DM.DataBase;
using Scenario.Modules.DM.DataModel;

namespace Scenario.Modules.DM.Application
{
	public class DocumentManagerHelper
	{
		private DocumentManagerDBHelper dmDBHelper;
		private DocumentHelper docHelper;

		public DocumentManagerHelper()
		{
			//Connecting to DB.
			dmDBHelper = new DocumentManagerDBHelper();

			//document Helper.
			docHelper  = new DocumentHelper (dmDBHelper);
		}

		public DocumentManagerHelper(DocumentManagerDBHelper dmDB)
		{
			//Connecting to DB.
			dmDBHelper = dmDB;

			//document Helper.
			docHelper  = new DocumentHelper (dmDBHelper);
		}



		public void CreateDocument(string sIdMaster, string sDocumentName, out string sIdSlave, out string sDocVersion)
		{
			XMLDocumentInstance docSlave = null;
			XMLDocumentInstance docMaster = null;
			bool bReExec=false;
			bool bManageVersion = true;
			
			XMLDocumentTemplate docTemplate=null;
			docTemplate= dmDBHelper.GetDocumentTemplate(sDocumentName);

			//Significa che il documento far da Master
			if ( sIdMaster.CompareTo (string.Empty ) == 0 )
			{
				docSlave = this.CreateDocument ( sDocumentName, "", "");
				docMaster = docSlave;
				bManageVersion = false;
			}
			else
			{
				//retrieving Master
				try
				{
					docMaster = dmDBHelper.GetDocumentInstance(sIdMaster);
				}
				catch ( Scenario.Modules.DM.DataBase.Exception.DMDBException )
				{
					throw;
				}

				//
				if ( docMaster.Name == sDocumentName )
				{
					docSlave = docMaster;
					bReExec = true;
				}
				else
				{
					XMLDocumentVersion lastVersion = docMaster.XMLDocumentVersions.GetLastVersion();

					//TODO: mfarina on 4/01/2008
					//Ho dovuto commentare in quanto in questo non era possibile gestire pi documenti
					//dello stesso tipo sul documento Padre. Al secondo tentativo di creazione del documento figlio
					//il codice sottostante recuperava la prima istanza del documento creato e lo versionava(se 
					//il flag di versione era valorizzato. In questo modo non gestisco pi la versione 
					//sul documento. 
					
					/*try
					{
						XMLDocumentInstance docSlaveTemp = (XMLDocumentInstance)lastVersion.XMLDocuments.Get (sDocumentName);
						bReExec = true;
						docSlave = dmDBHelper.GetDocumentInstance( docSlaveTemp.Id );
					}
					catch ( DocumentNotFound)
					{*/
						try
						{
							//definire la migrazione.
							docSlave = this.CreateDocument (sDocumentName, docMaster.Id,lastVersion.Id);
							lastVersion.XMLDocuments.Add (docSlave);
							dmDBHelper.SaveDocumentInstance (docMaster, DocumentManagerDBHelper.SaveMode.DOCUMENT);

							bManageVersion = false;

						}
                        catch (Scenario.Modules.DM.DataModel.Exception.DocumentNotFound)
						{
							throw;
						}
					//}
				}

				docMaster.LastDocument = sDocumentName;
				dmDBHelper.SaveDocumentInstance (docMaster, DocumentManagerDBHelper.SaveMode.DOCUMENT);
			}

			if ( docSlave.ManageVersion == XMLDocumentBase.YES_MANAGEVERSION && bManageVersion ) 
			{
				docHelper.AddVersion (docSlave);
				dmDBHelper.SaveDocumentInstance (docSlave, DocumentManagerDBHelper.SaveMode.DOCUMENT);
			}

			if ( docSlave.CreationPolicy == XMLDocumentBase.MIGRATE_CREATIONPOLICY )
			{
				//Migrazione.................
				XMLDocumentMigrate migrate = null;
				for ( int i=0; i<docMaster.XMLDocumentMigrates.Count;i++)
				{
					migrate = (XMLDocumentMigrate)docMaster.XMLDocumentMigrates[i];

					if ( ( migrate.To == docSlave.Name ) && 
						( (bReExec && (migrate.ReExec == "YES") ) || (!bReExec && (migrate.ReExec == "NO"))) ) 
					{
						//retrieving File.

						XMLDocumentVersion versionRef = null;
						XMLDocumentInstance doc = null;
						for ( int j=docMaster.XMLDocumentVersions.Count; j>0 && (doc == null); j--)
						{
							versionRef = (XMLDocumentVersion)docMaster.XMLDocumentVersions[j-1];
							if ( migrate.From == docMaster.Name ) 
							{
								doc = docMaster;
							}
							else
							{
								try
								{
									string sDocName = ((XMLDocumentInstance)versionRef.XMLDocuments.Get(migrate.From)).Id;
									doc = dmDBHelper.GetDocumentInstance(sDocName);
								}
                                catch (Scenario.Modules.DM.DataModel.Exception.DocumentNotFound)
								{
								}
							}
						}

						if ( doc != null )
						{
							XMLDocumentVersion docVersion = doc.XMLDocumentVersions.GetLastVersion();
							XMLDocumentVersion docSlaveVersion = docSlave.XMLDocumentVersions.GetLastVersion();

							string sFileFrom = docHelper.GetInstanceFileName (doc.Id, docVersion.Id, doc.XMLDocumentFile.Id, doc.XMLDocumentFile.Type);
							string sFileTo = docHelper.GetInstanceFileName (docSlave.Id, docSlaveVersion.Id, docSlave.XMLDocumentFile.Id, docSlave.XMLDocumentFile.Type);
	 
							switch ( migrate.Type )
							{
								case XMLDocumentMigrate.EMBEDDED_TYPE:

									docHelper.MigrateEmbedded(sFileFrom, sFileTo, migrate.File);
									break;

								case XMLDocumentMigrate.XSLT_TYPE:
									break;

								case XMLDocumentMigrate.DOTNET_TYPE:
									break;

								case XMLDocumentMigrate.COPY_TYPE:
									DocumentHelper.Copy (sFileFrom, sFileTo);
									break;

							}


							XMLDocumentFile docFile = null;
							for ( int j=0; j<docVersion.XMLDocumentAttachFiles.Count;j++)
							{
								docFile = (XMLDocumentFile)docVersion.XMLDocumentAttachFiles[j];

								if ( docFile.MigratePolicy == XMLDocumentFile.DOCUMENT_MIGRATE_POLICY )
								{
									XMLDocumentFile newFile = new XMLDocumentFile();
									newFile.Parse ( docFile.GetXML() );

									docSlaveVersion.XMLDocumentAttachFiles.AddDocumentFile ( newFile );

									string sFileOldVersion = docHelper.GetInstanceFileName(doc.Id, docVersion.Id, docFile.Id, docFile.Type);
									string sFileNewVersion = docHelper.GetInstanceFileName(docSlave.Id, docSlaveVersion.Id, newFile.Id, newFile.Type,docSlave.Name);

									try
									{
										System.IO.File.Copy(sFileOldVersion, sFileNewVersion, true);
									}
									catch ( System.Exception )
									{
										//TODO: Impossible trasferire il file.
									}
								}
							}
							try
							{
								dmDBHelper.SaveDocumentInstance(docSlave, DocumentManagerDBHelper.SaveMode.DOCUMENT);
							}
                            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException)
							{
								throw;
							}
						}
					}
				}
			}

			sIdSlave = docSlave.Id;
			sDocVersion = docSlave.XMLDocumentVersions.GetLastVersion().Id;
		}

		public void CreateDocuments(string sIdMaster, string sDocumentName, int iNumberOfInstance, out string[] sIdSlave, out string[] sDocVersion)
		{
			sIdSlave = new string[iNumberOfInstance];
			sDocVersion= new string[iNumberOfInstance];;
			for (int i=0; i< iNumberOfInstance; i++)
			{
				string sIdsl=string.Empty;
				string sdocVer=string.Empty;
			
				this.CreateDocument(sIdMaster,sDocumentName,out sIdSlave[i],out sDocVersion[i]);				
				
			}
		}
		

		//1- Retrieve Document Template from DB.
		//2- Creazione del Documento
		//3- Salvataggio del documento.
		private XMLDocumentInstance CreateDocument(string sDocumentName, string sIdMaster, string sVerMaster)
		{
			XMLDocumentTemplate docTemplate = null;
			
			try
			{
				//Retrieving Document Template
				docTemplate = dmDBHelper.GetDocumentTemplate(sDocumentName);

			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException)
			{
				throw;
			}

			//Creating Master Document instance.
			XMLDocumentInstance docInstance = new XMLDocumentInstance();
			docInstance.Parse (docTemplate.GetXML());
			
			if ( sIdMaster.CompareTo ( string.Empty ) != 0 )
			{
				docInstance.IdMaster = sIdMaster;
				docInstance.VersionMaster = sVerMaster;
			}

			bool recycled= false;
			docInstance.Id = docHelper.ResolveId (docTemplate.IdFormat, docInstance,out recycled);
			Scenario.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttribute xmlAttribute = 
                new Scenario.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttribute("DocumentStorage", "String", "NONE");
			xmlAttribute.Value = docHelper.ResolveSavingMode (docTemplate, sIdMaster).Replace ("\\", "\\\\");
			docInstance.XMLAttributes.Add ( xmlAttribute );
			if (recycled)
			{
				ArrayList docs= this.dmDBHelper.DeleteAllChild(docInstance.Id);
				this.DeleteAllChildFromFS(docs);
				this.dmDBHelper.DeleteDocs(docs);
			}
			docHelper.AddVersion (docInstance);

			try
			{

				docInstance.LastDocument = sDocumentName;
				dmDBHelper.SaveDocumentInstance(docInstance, DocumentManagerDBHelper.SaveMode.DOCUMENT);
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException)
			{
				throw;
			}

			return docInstance;
		}

		
		private XMLDocumentInstance GetDocumentInstance(string sIdMaster, string sDocName)
		{
			XMLDocumentInstance docMaster = new XMLDocumentInstance();
			try
			{
				//Retrieving Document Template
				docMaster = dmDBHelper.GetDocumentInstance(sIdMaster);
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException)
			{
				return null;
			}

			if ( docMaster.Name == sDocName )
			{
				return docMaster;
			}

			XMLDocumentVersion lastVersion = docMaster.XMLDocumentVersions.GetLastVersion();
			XMLDocumentInstance docSlave = null;
			try
			{
				XMLDocumentInstance docSlaveTemp = (XMLDocumentInstance)lastVersion.XMLDocuments.Get (sDocName);
				docSlave = dmDBHelper.GetDocumentInstance( docSlaveTemp.Id );
			}
            catch (Scenario.Modules.DM.DataModel.Exception.DocumentNotFound)
			{
				return null;
			}

			return docSlave;
		}


		public XMLDocumentInstance GetLastDocument(string sIdMaster)
		{
			XMLDocumentInstance docMaster = new XMLDocumentInstance();
			try
			{
				//Retrieving Document Template
				docMaster = dmDBHelper.GetDocumentInstance(sIdMaster);
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException)
			{
				return null;
			}

			string sDocName = docMaster.LastDocument;

			if ( docMaster.Name == sDocName )
			{
				return docMaster;
			}
		
			XMLDocumentVersion lastVersion = docMaster.XMLDocumentVersions.GetLastVersion();
			XMLDocumentInstance docSlave = null;
			try
			{
				XMLDocumentInstance docSlaveTemp = (XMLDocumentInstance)lastVersion.XMLDocuments.Get (sDocName);
				docSlave = dmDBHelper.GetDocumentInstance( docSlaveTemp.Id );
			}
            catch (Scenario.Modules.DM.DataModel.Exception.DocumentNotFound)
			{
				return null;
			}

			return docSlave;
		}


		private void DeleteAllChildFromFS(ArrayList childList)
		{
			/*
			string sPath= this.dmDBHelper.GetInstanceFilePathNoFile();
			foreach (string sId in childList)
			{
				XMLDocumentInstance xmlInst = this.dmDBHelper.LoadDocumentInstance(sId,true);
				XMLDocumentTemplate xmltemplate=this.dmDBHelper.GetDocumentTemplate(xmlInst.Name);
				string sPathRes=this.docHelper.ResolveSavingMode(xmltemplate,"");
				System.IO.File.Delete(sPath+sPathRes+sId+".xml");
				System.IO.File.Delete(this.docHelper.GetInstanceFileName(sId));
			}
			*/
		}

		
		
		#region Reading and writing Data File for XML File.
		public string GetXMLDataFromLastDocument(string sIdMaster)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetLastDocument(sIdMaster);
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			string sIdFile = docHelper.GetInstanceFileName(document.Id , lastVersion.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);

			try
			{
				System.IO.StreamReader myFile =	new System.IO.StreamReader(sIdFile);
				string sXML = myFile.ReadToEnd();
				int Last = sXML.IndexOf("?>");
				if ( Last >=0 )
				{
					sXML=sXML.Substring(Last+2);
				}
				myFile.Close();
				myFile = null;

				return sXML;
			}
			catch ( System.Exception err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}
		}

		public string GetXMLData(string sIdMaster, string sDocName)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetDocumentInstance(sIdMaster, sDocName);
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			string sIdFile = docHelper.GetInstanceFileName(document.Id , lastVersion.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);

			try
			{
				System.IO.StreamReader myFile =	new System.IO.StreamReader(sIdFile);
				string sXML = myFile.ReadToEnd();
				int Last = sXML.IndexOf("?>");
				if ( Last >=0 )
				{
					sXML=sXML.Substring(Last+2);
				}
				myFile.Close();
				myFile = null;

				return sXML;
			}
			catch ( System.Exception err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}
		}

		public string RemoveRowFromGrid(string sIdMaster, string sDocName, string sGrid, int iRow)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetDocumentInstance(sIdMaster,sDocName );
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			string sIdFile = docHelper.GetInstanceFileName (document.Id, lastVersion.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);

			
			try
			{
				XmlDocument MyModule = new XmlDocument();
				MyModule.Load(sIdFile);

				XmlNode node= MyModule.SelectSingleNode ("Module/Matrixes/" + sGrid);

				if ( iRow <= node.ChildNodes.Count )
				{
					node.RemoveChild ( node.ChildNodes[iRow-1] );
				}

				//salva le modifiche introdotte
				MyModule.Save(sIdFile);
				return MyModule.InnerXml;

			}
			catch(System.Exception err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}				
						
		}

		public string AddRowToGrid(string sIdMaster, string sDocName, string sGrid, string sXMLData)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetDocumentInstance(sIdMaster,sDocName );
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			string sIdFile = docHelper.GetInstanceFileName (document.Id, lastVersion.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);

			
			try
			{
				XmlDocument MyModule = new XmlDocument();
				MyModule.Load(sIdFile);

				XmlNode gridNode = MyModule.SelectSingleNode ("Module/Matrixes/" + sGrid);

				XmlDocument MyDataList = new XmlDocument();	
				MyDataList.LoadXml(sXMLData);
				XmlNode rootDataList=MyDataList.FirstChild;

				// scorre tutti i nodi di datalist
				for (int i=0; i< rootDataList.ChildNodes.Count;++i)
				{
					XmlNode nodeRow = MyModule.ImportNode( rootDataList.ChildNodes[i] , true);
					gridNode.AppendChild (nodeRow);

				}

				//salva le modifiche introdotte
				MyModule.Save(sIdFile);

				return MyModule.InnerXml;
			}
			catch(System.Exception err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}				
						
		}

		public void SetXMLData(string sIdMaster, string sDocName,string sXMLData)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetDocumentInstance(sIdMaster,sDocName );
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			string sIdFile = docHelper.GetInstanceFileName (document.Id, lastVersion.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);

			
			try
			{
				XmlDocument MyModule = new XmlDocument();
				MyModule.Load(sIdFile);

				XmlDocument MyDataList = new XmlDocument();	
				MyDataList.LoadXml(sXMLData);

				// variabli per l'interpretazione del datalist
				XmlNode ModuleElement;
				XmlNode rootDataList;

				rootDataList=MyDataList.FirstChild;


				// scorre tutti i nodi di datalist
				for (int i=0; i< rootDataList.ChildNodes.Count;++i)
				{
					XmlNode root;
					root=rootDataList.ChildNodes[i];
					// selsziona il tipo di nodo e la modifica da effetture
					if (root.Attributes["IsMatrix"].Value=="NO")
					{	
									
						//cerco in myModule l'elemento e modifico il valore
						ModuleElement=MyModule.SelectSingleNode("Module/Fields/" + root.Attributes["Name"].Value);
						//ModuleElement.Value=root.Value;
						ModuleElement.InnerText=root.InnerText;

					}
					else
					{
						//cerco in myModule l'elemento della matrice e modifico il valore
						ModuleElement=MyModule.SelectSingleNode("Module/Matrixes/" + root.Attributes["Name"].Value + "/Row" + root.Attributes["RowNum"].Value + "/Col" + root.Attributes["ColNum"].Value);
						//ModuleElement.Value=root.Value;
						ModuleElement.InnerText=root.InnerText;
					}

				}

				//salva le modifiche introdotte
				MyModule.Save(sIdFile);
				MyModule = null;

			}
			catch(System.Exception err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}				
						
		}

		#endregion

		#region Attaches
		public XMLDocumentAttachFiles GetAttaches(string sIdMaster, string sDocumentType)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetDocumentInstance(sIdMaster, sDocumentType);
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			XMLDocumentFile docFile = null;
			for ( int i=0; i<lastVersion.XMLDocumentAttachFiles.Count;i++)
			{
				docFile = (XMLDocumentFile)lastVersion.XMLDocumentAttachFiles[i];
				docFile.Download = docHelper.GetInstanceFileName(document.Id, lastVersion.Id, docFile.Id, docFile.Type);
			}
			return lastVersion.XMLDocumentAttachFiles;
		}

		//1 - File doesn't exist.
		//2 - File locked.
		public void RemoveAttach(string sIdMaster, string sDocumentType, string sAttach)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetDocumentInstance(sIdMaster, sDocumentType);
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			XMLDocumentFile xmlFile=null;
			for ( int i=0; i<lastVersion.XMLDocumentAttachFiles.Count; i++)
			{
				xmlFile = (XMLDocumentFile)lastVersion.XMLDocumentAttachFiles[i];
				if ( xmlFile.Name == sAttach)
				{
					if ( xmlFile.Status == XMLDocumentFile.LOCK_STATUS ) 
					{
						throw new Exception.DocumentManagerException (2, "File already locked" );
					}
					else
					{
						string sFileName = docHelper.GetInstanceFileName(document.Id, lastVersion.Id, xmlFile.Id, xmlFile.Type);
						try
						{
							System.IO.File.Delete (sFileName);
						}
						catch ( System.Exception err)
						{
							throw new Exception.DocumentManagerException (err.Message );
						}
						lastVersion.XMLDocumentAttachFiles.RemoveAt (i);
						dmDBHelper.SaveDocumentInstance (document, DocumentManagerDBHelper.SaveMode.DOCUMENT);

						return;
					}
				}
			}
			throw new Exception.DocumentManagerException (1, "File doesn't exist" );
		}

		//1 - File doesn't exist.
		//2 - File already locked.
		public void LockAttach( string sIdDoc,string sAttach)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = dmDBHelper.GetDocumentInstance(sIdDoc);
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			XMLDocumentFile xmlFile = lastVersion.XMLDocumentAttachFiles.GetFileByName(sAttach);
			if ( xmlFile == null )
			{
				throw new Exception.DocumentManagerException (1, "File doesn't exist" );
			}

			if ( xmlFile.Status == XMLDocumentFile.LOCK_STATUS )
			{
				throw new Exception.DocumentManagerException (2, "File already locked" );
			}

			xmlFile.Status = XMLDocumentFile.LOCK_STATUS;
			dmDBHelper.SaveDocumentInstance (document, DocumentManagerDBHelper.SaveMode.DOCUMENT);
		}

		//1 - File doesn't exist.
		//2 - File already unlocked.
		public void UnlockAttach( string sIdDoc,string sAttach)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = dmDBHelper.GetDocumentInstance(sIdDoc);
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			XMLDocumentFile xmlFile = lastVersion.XMLDocumentAttachFiles.GetFileByName(sAttach);
			if ( xmlFile == null )
			{
				throw new Exception.DocumentManagerException (1, "File doesn't exist" );
			}

			if ( xmlFile.Status == XMLDocumentFile.UNLOCK_STATUS )
			{
				throw new Exception.DocumentManagerException (2, "File already unlocked" );
			}

			xmlFile.Status = XMLDocumentFile.UNLOCK_STATUS;
			dmDBHelper.SaveDocumentInstance (document, DocumentManagerDBHelper.SaveMode.DOCUMENT);
		}


		public void AddAttach(string sIdMaster, string sDocName, XMLDocumentFile docFile, string sPath)
		{
			XMLDocumentInstance document=null;
			XMLDocumentVersion lastVersion = null;
			try
			{
				document = this.GetDocumentInstance (sIdMaster,sDocName );
				lastVersion = document.XMLDocumentVersions.GetLastVersion();
			}
            catch (Scenario.Modules.DM.DataBase.Exception.DMDBException err)
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			XMLDocumentAttachFiles xmlAttaches = lastVersion.XMLDocumentAttachFiles;

			XMLDocumentFile xmlFile = xmlAttaches.GetFileByName(docFile.Name);
			if ( xmlFile != null )
			{
				throw new Exception.DocumentManagerException (1, "File Already exists.");
			}
			
			string sFileTarget = docHelper.GetInstanceFileName (document.Id , lastVersion.Id, docFile.Id,docFile.Type);
			
			try
			{
				System.IO.File.Move (sPath, sFileTarget);
			}
			catch ( System.Exception err) 
			{
				throw new Exception.DocumentManagerException (err.Message );
			}

			xmlAttaches.Add(docFile);

			dmDBHelper.SaveDocumentInstance (document, DocumentManagerDBHelper.SaveMode.DOCUMENT);
		}

		#endregion
	}
}
