using System;
using System.Collections;
using System.Xml;
using System.Data;
using System.IO;

using MySql.Data.MySqlClient;
using Scenario.Modules.DM.DataModel;

namespace Scenario.Modules.DM.DataBase
{
	public class DocumentManagerDBHelper : bOS.DataBase.DataBaseManager
	{
		string sDBDirectory = string.Empty;

		private const string sDMPath = "\\DM\\";
		private const string sDocTemplateDirectory = "Template\\";
		private const string sDocTemplateFileDirectory = "Files\\";

		private const string sDocInstanceDirectory = "Instance\\";
		private const string sDocInstanceFileDirectory = "Files\\";

		private const string sDocClosedDirectory = "Complete\\";
		private const string sDocClosedFileDirectory = "Files\\";

		private const string sTempDirectory = "Temporary\\";

		private static DocumentCounter docCounter = null;

	
		#region SQL

		#region Protocollo
		private const string SELECT_ID= "Select counter from DM_ID_INSTANCES WHERE id_instance=@id";
		private const string SELECT_YEAR= "Select year from DM_ID_INSTANCES WHERE id_instance=@id";
		private const string INSERT_ID= "Insert into DM_ID_INSTANCES(id_instance, counter, year) VALUES (@id,@count,@y)";

		private const string UPDATE_ID= "UPDATE DM_ID_INSTANCES SET counter=@count WHERE id_instance=@id";
		private const string UPDATE_ID_NEWYEAR= "UPDATE DM_ID_INSTANCES SET counter=1, year=@y WHERE id_instance=@id";

		private const string SELECT_ID_RECYCLE= "Select id_num from DM_IDINSTANCES_TO_RECYCLE WHERE id_instance=@id and recycled=\'false\'";
		private const string UPDATE_RECYCLED= "UPDATE DM_IDINSTANCES_TO_RECYCLE SET recycled=\'true\' WHERE id_instance=@id and id_num=@num";
		#endregion

		
		//modified by e.leano for new version of DM: step 2
		private const string INSERT_INSTANCE ="INSERT INTO DM_DOCUMENTS(Id,Type,Name,Creation_policy, Manage_version , IdMaster ,Version_Master,LastDocument,Migrates, Grants) VALUES(@id,@type,@name,@cp ,@mv,@IdMaster,@Version_Master,@lastDoc,@Migr,@grants) ";
		private const string INSERT_FILE ="INSERT INTO dm_document_files(id,name,description,status,type,migration_policy,download,id_doc) VALUES (@id,@name,@des,@stat,@type,@mp,@down,@id_doc)";
		private const string INSERT_DOC_CHILD="INSERT INTO dm_doc_childs(id_doc,version,id_child) VALUES (@id,@ver,@child)";
		private const string INSERT_VERSION= "INSERT INTO dm_document_versions (Id_doc,Description,CreationDate,Attaches,Grants,Version,Logs,Reports)VALUES(@id_doc,@des,@cDate,@attaches,@grants,@version,@logs,@reports);";
		private const string SELECT_FILE= "SELECT * FROM dm_document_files WHERE id_doc=@id";
		private const string SELECT_ATTRIBUTES_DOCUMENT="SELECT * FROM dm_documents WHERE Id=@id";
		private const string SELECT_DOCUMENT_LIST="SELECT * FROM dm_documents #WHERE# order by id";
		private const string SELECT_MIGRATES= "SELECT Migrates FROM dm_documents WHERE Id=@id";
		private const string SELECT_GRANTS= "SELECT Grants FROM dm_documents WHERE Id=@id";
		private const string SELECT_CHILDS ="SELECT id_child FROM dm_doc_childs WHERE id_doc=@id and version=@version";
		private const string SELECT_ALL_CHILDS ="SELECT id_child FROM dm_doc_childs WHERE id_doc=@id";
		
		private const string DELETE_CHILDS ="DELETE FROM dm_doc_childs WHERE id=@id";
		private const string DELETE_DOC="DELETE FROM DM_DOCUMENTS WHERE id_doc=@id";
		private const string DELETE_VER="DELETE FROM dm_document_versions WHERE id_doc=@id";
		private const string DELETE_FILE="DELETE FROM dm_document_files WHERE id=@id";

	
	
		private const string SELECT_VERSION="Select Version FROM dm_document_versions where id_doc=@id";
		private const string SELECT_VERSION_I="Select * FROM dm_document_versions where id_doc=@id and Version=@version";

		private const string SELECT_LOGS= "SELECT Logs FROM dm_documents WHERE Id=@id";
		private const string SELECT_REPORTS= "SELECT Reports FROM dm_documents WHERE Id=@id";

		private const string SELECT_ATTACHES= "SELECT Attaches FROM dm_documents WHERE Id=@id";

		private const string SELECT_ATTRIBUTES="Select name,direction,type,value from dm_attributes where docId=@id AND Version=@ver";
		private const string INSERT_ATTRIBUTES ="INSERT INTO dm_attributes(DOcId,Version,Name,Direction,Type,Value) VALUES(@id,@ver,@name,@dir,@type, @val) ";
		
		#endregion

		System.Threading.Mutex mtxIdTable = new System.Threading.Mutex();

		public enum SaveMode
		{
			DOCUMENT,
			LAST_VERSION,
			ALL_VERSION
		};

		#region Constructors
		
		private void Initialize()
		{
			WAPI.Response response = new WAPI.Response();
			string sScenarioDB = WAPI.Session.GetDirectory ("WF_DMPATH", ref response);
			if ( response.iCode != 0 )
			{
				sScenarioDB = WAPI.Session.GetScenarioDBDirectory (ref response);
			}
			sDBDirectory = sScenarioDB + sDMPath;

			this.ConnectionString = WAPI.DataTypes.GetValue ("DataBase", "ConnectionString");
		}

		private void Initialize(string sDataType, string sDataKey)
		{
			WAPI.Response response = new WAPI.Response();
			string sScenarioDB = WAPI.Session.GetDirectory ("WF_DMPATH", ref response);
			if ( response.iCode != 0 )
			{
				sScenarioDB = WAPI.Session.GetScenarioDBDirectory (ref response);
			}

			sDBDirectory = sScenarioDB + sDMPath;
			
			this.ConnectionString = WAPI.DataTypes.GetValue (sDataType, sDataKey);
		}

		public DocumentManagerDBHelper() : base()
		{
			this.Initialize ();
		}


		public DocumentManagerDBHelper(string sDataType, string sDataKey) : base()
		{
			this.Initialize (sDataType,sDataKey);
		}

		
		public DocumentManagerDBHelper(string sPath) : base()
		{
			this.Initialize("DataBase", "ConnectionString");
			sDBDirectory = sPath + sDMPath;
		}


		public DocumentManagerDBHelper(string sPath, string sDataType, string sDataKey) : base()
		{
			this.Initialize(sDataType, sDataKey);
			sDBDirectory = sPath + sDMPath;
		}

		
		public long GetDocumentProgressiveFromDB(string sDocId, out bool recycled)
		{
			mtxIdTable.WaitOne();
			long prog=0;
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_ID_RECYCLE);
			myCommand.Parameters.AddWithValue ("@id",sDocId);
			object results= this.ExecuteScalar(myCommand);
			if (results!=null)
			{
				prog = long.Parse(results.ToString());
				myCommand = new MySqlCommand(DocumentManagerDBHelper.UPDATE_RECYCLED);
                myCommand.Parameters.AddWithValue("@num", prog.ToString());
                myCommand.Parameters.AddWithValue("@id", sDocId);
				this.ExecuteNoQuery(myCommand);
				recycled=true;
			}
			else
			{
				myCommand = new MySqlCommand(DocumentManagerDBHelper.SELECT_ID);
                myCommand.Parameters.AddWithValue("@id", sDocId);
				results= this.ExecuteScalar(myCommand);
				
				if (results==null)
				{
					myCommand = new MySqlCommand(DocumentManagerDBHelper.INSERT_ID);
                    myCommand.Parameters.AddWithValue("@id", sDocId);
                    myCommand.Parameters.AddWithValue("@count", "1");
                    myCommand.Parameters.AddWithValue("@y", DateTime.Now.Year.ToString());
					this.ExecuteNoQuery(myCommand);
					prog=1;
				}
				else
				{
					prog = long.Parse(results.ToString());
				
				}
				long upProg=prog+1;

				myCommand = new MySqlCommand(DocumentManagerDBHelper.UPDATE_ID);
                myCommand.Parameters.AddWithValue("@count", upProg.ToString());
                myCommand.Parameters.AddWithValue("@id", sDocId);
				this.ExecuteNoQuery(myCommand);
				recycled=false;
			}
				
				mtxIdTable.ReleaseMutex();
			
			return prog;
		}

		public long GetDocumentYearProgressive(string sDocId)
		{
			mtxIdTable.WaitOne();
			int year = DateTime.Now.Year;
			
			MySqlCommand myCommand = new MySqlCommand(DocumentManagerDBHelper.SELECT_YEAR);
            myCommand.Parameters.AddWithValue("@id", sDocId);
            myCommand.Parameters.AddWithValue("@year", year);
			object results= this.ExecuteScalar(myCommand);
			
			if (results == null)
			{
				myCommand = new MySqlCommand(DocumentManagerDBHelper.INSERT_ID);
                myCommand.Parameters.AddWithValue("@id", sDocId);
                myCommand.Parameters.AddWithValue("@count", 1);
                myCommand.Parameters.AddWithValue("@y", year.ToString());
				
				this.ExecuteNoQuery(myCommand);
			}
			else
			{
				year = int.Parse(results.ToString());
				if (year < DateTime.Now.Year)
				{
					myCommand = new MySqlCommand(DocumentManagerDBHelper.UPDATE_ID_NEWYEAR);
                    myCommand.Parameters.AddWithValue("@id", sDocId);
                    myCommand.Parameters.AddWithValue("@y", DateTime.Now.Year.ToString());
					this.ExecuteNoQuery(myCommand);
				}
			}

			mtxIdTable.ReleaseMutex();
			bool recycled= false;

			return this.GetDocumentProgressiveFromDB(sDocId, out recycled);
		}

		
		public static long GetDocumentProgressive()
		{
			string sScenarioDB = string.Empty;
			
			WAPI.Response response = new WAPI.Response();
			sScenarioDB = WAPI.Session.GetDirectory("WF_DMPATH", ref response);
			if ( response.iCode != 0 )
			{
				sScenarioDB = WAPI.Session.GetScenarioDBDirectory(ref response);
			}
			
			if ( docCounter == null )
			{
				docCounter = new DocumentCounter ( sScenarioDB + sDMPath );
			}

			return docCounter.Increment(sScenarioDB + sDMPath);
		}

		public static long GetDocumentProgressive(string sDocument)
		{
			
				WAPI.Response response = new WAPI.Response();
				string sScenarioDB = WAPI.Session.GetScenarioDBDirectory(ref response);
				if ( response.iCode != 0 )
				{
					//TODO::
				}
			if ( docCounter == null )
			{
				docCounter = new DocumentCounter ( sScenarioDB + sDMPath, sDocument );
			}

			return docCounter.Increment(sScenarioDB + sDMPath, sDocument);
		}

		#endregion

		#region Document Directories
		public string GetTempDirectory()
		{
			return this.sDBDirectory + DocumentManagerDBHelper.sDocTemplateDirectory;
		}
		public string GetInstanceDirectory()
		{
			return this.sDBDirectory + DocumentManagerDBHelper.sDocInstanceDirectory;
		}

		public string GetTemplateFilePath()
		{
			return this.sDBDirectory + DocumentManagerDBHelper.sDocTemplateDirectory + sDocTemplateFileDirectory;
		}

		public string GetInstanceFilePath()
		{
			return this.sDBDirectory + DocumentManagerDBHelper.sDocInstanceDirectory + sDocInstanceFileDirectory;
		}

		#endregion

		
		#region Save Methods

		public string GetDocPath(XMLDocumentInstance docInstance)
		{
			//
			string sDocPath  = this.sDBDirectory + DocumentManagerDBHelper.sDocInstanceDirectory;

			Scenario.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttribute xmlAttribute = null;
			try
			{
				xmlAttribute = docInstance.XMLAttributes.Get ("DocumentStorage");
			}
			catch {}

			if ( xmlAttribute != null )
			{
				sDocPath+=xmlAttribute.Value;
			}

			if (!Directory.Exists(sDocPath))
			{
				Directory.CreateDirectory(sDocPath);
			}
			string sPath=sDocPath + "Files";
			if (!Directory.Exists(sPath))
			{
				Directory.CreateDirectory(sPath);
			}

			return sDocPath;
		}
		public void SaveDocumentInstance (XMLDocumentInstance docInstance)
		{
			string sDocPath  = this.GetDocPath (docInstance);
			sDocPath+=docInstance.Id + ".xml";

			try
			{
				//Save 
				docInstance.Save (sDocPath);

				
			}
			catch ( System.IO.IOException )
			{
				throw new Exception.DMDBException("Impossible to save Document Instance");
			}
		}

		public void SaveDocumentInstance (XMLDocumentInstance docInstance, SaveMode iSaveMode)
		{
			//qui v definita la transazione
			mtxIdTable.WaitOne();
				
			MySqlCommand myCommand = new MySqlCommand(DocumentManagerDBHelper.INSERT_INSTANCE);
            myCommand.Parameters.AddWithValue("@id", docInstance.Id);
            myCommand.Parameters.AddWithValue("@type", docInstance.Type);
            myCommand.Parameters.AddWithValue("@name", docInstance.Name);
            myCommand.Parameters.AddWithValue("@cp", docInstance.CreationPolicy);
            myCommand.Parameters.AddWithValue("@mv", docInstance.ManageVersion);
            myCommand.Parameters.AddWithValue("@IdMaster", docInstance.IdMaster);
            myCommand.Parameters.AddWithValue("@Version_Master", docInstance.VersionMaster);
            myCommand.Parameters.AddWithValue("@lastDoc", docInstance.LastDocument);
            myCommand.Parameters.AddWithValue("@Migr", docInstance.XMLDocumentMigrates.GetXML());
            myCommand.Parameters.AddWithValue("@grants", docInstance.XMLGrants.GetXML());

			bool bSaveDocumentTreeAndProperties = true;
			try
			{
				this.ExecuteNoQuery(myCommand);
			}
			catch ( MySql.Data.MySqlClient.MySqlException mySqlErr)
			{
				switch ( mySqlErr.Number )
				{
						/*case ByteFX.Data.MySqlClient.MySqlError..DuplicateKey:
						case ByteFX.Data.MySqlClient.MySqlError.DuplicateKeyEntry:

							//XMLDocumentInstance docInstanceDB = this.LoadDocumentInstance(docInstance.Id, false);
							//docInstance.AddAttribute ( "DocumentStorage", docInstanceDB.GetAttribute ();

							bSaveDocumentTreeAndProperties = false;
							break;
						*/
					default: //serious error
						bSaveDocumentTreeAndProperties = false;
						break;
				}
			}
			catch ( System.Exception err)
			{
				bSaveDocumentTreeAndProperties = false;
			}
									  
			
			if ( bSaveDocumentTreeAndProperties )
			{
				try
				{
					//inserimento nella tabella dm_file
					myCommand = new MySqlCommand(DocumentManagerDBHelper.INSERT_FILE);
                    myCommand.Parameters.AddWithValue("@id", docInstance.XMLDocumentFile.Id);
                    myCommand.Parameters.AddWithValue("@name", docInstance.XMLDocumentFile.Name);
                    myCommand.Parameters.AddWithValue("@des", docInstance.XMLDocumentFile.Description);
                    myCommand.Parameters.AddWithValue("@stat", docInstance.XMLDocumentFile.Status);
                    myCommand.Parameters.AddWithValue("@type", docInstance.XMLDocumentFile.Type);
                    myCommand.Parameters.AddWithValue("@mp", docInstance.XMLDocumentFile.MigratePolicy);
                    myCommand.Parameters.AddWithValue("@id_doc", docInstance.Id);
					this.ExecuteNoQuery(myCommand);

					//Inserimento versioni
					foreach(XMLDocumentVersion xVer in docInstance.XMLDocumentVersions)
					{
						myCommand= new MySqlCommand(DocumentManagerDBHelper.INSERT_VERSION);
                        myCommand.Parameters.AddWithValue("@id_doc", docInstance.Id);
                        myCommand.Parameters.AddWithValue("@des", xVer.Description);
                        myCommand.Parameters.AddWithValue("@cDate", xVer.CreationDate);
                        myCommand.Parameters.AddWithValue("@attaches", xVer.XMLDocumentAttachFiles.GetXML());
                        myCommand.Parameters.AddWithValue("@grants", docInstance.XMLGrants.GetXML());
                        myCommand.Parameters.AddWithValue("@version", xVer.Id);
                        myCommand.Parameters.AddWithValue("@logs", xVer.XMLLogs.GetXML());
                        myCommand.Parameters.AddWithValue("@reports", xVer.XMLReports.GetXML());
						this.ExecuteNoQuery(myCommand);
						
						//inserimento figli a livello di versione
						foreach (XMLDocumentInstance xdoc in xVer.XMLDocuments)
						{
							myCommand= new MySqlCommand(DocumentManagerDBHelper.INSERT_DOC_CHILD);
                            myCommand.Parameters.AddWithValue("@id", docInstance.Id);
                            myCommand.Parameters.AddWithValue("@ver", xVer.Id);
                            myCommand.Parameters.AddWithValue("@child", xdoc.Id);
							this.ExecuteNoQuery(myCommand);
                            foreach (Scenario.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttribute xmlAtt in xVer.XMLAttributes)
							{
								myCommand=new MySqlCommand(DocumentManagerDBHelper.INSERT_ATTRIBUTES);
                                myCommand.Parameters.AddWithValue("@id", docInstance.Id);
                                myCommand.Parameters.AddWithValue("@ver", xVer.Id);
                                myCommand.Parameters.AddWithValue("@type", xmlAtt.Type);
                                myCommand.Parameters.AddWithValue("@name", xmlAtt.Name);
                                myCommand.Parameters.AddWithValue("@dir", xmlAtt.Direction);
                                myCommand.Parameters.AddWithValue("@val", xmlAtt.Value);
								this.ExecuteNoQuery(myCommand);
							}

						}
					}

					foreach (Scenario.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttribute xmlAtt in docInstance.XMLAttributes)
					{
						myCommand=new MySqlCommand(DocumentManagerDBHelper.INSERT_ATTRIBUTES);
                        myCommand.Parameters.AddWithValue("@id", docInstance.Id);
                        myCommand.Parameters.AddWithValue("@ver", "1");
                        myCommand.Parameters.AddWithValue("@type", xmlAtt.Type);
                        myCommand.Parameters.AddWithValue("@name", xmlAtt.Name);
                        myCommand.Parameters.AddWithValue("@dir", xmlAtt.Direction);
                        myCommand.Parameters.AddWithValue("@val", xmlAtt.Value);
						this.ExecuteNoQuery(myCommand);
					}
				}
				catch (bOS.DataBase.DataBaseManagerException err)
				{
					string s = string.Empty;
				}
			}

			try
			{
				this.SaveDocumentInstance ( docInstance);

				switch ( iSaveMode ) 
				{
					case SaveMode.DOCUMENT:
						break;

					case SaveMode.LAST_VERSION:
						XMLDocumentVersion lastVersion = docInstance.XMLDocumentVersions.GetLastVersion();
						//this.SaveInstanceFile ( docInstance.Id, lastVersion.Id, docInstance.XMLDocumentFile);
						this.SaveAttachFiles (docInstance.Id,lastVersion.Id, lastVersion.XMLDocumentAttachFiles);
						break;

					case SaveMode.ALL_VERSION:
						XMLDocumentVersion version = null;
						for ( int i=0; i<docInstance.XMLDocumentVersions.Count; i++)
						{
							version = (XMLDocumentVersion)docInstance.XMLDocumentVersions[i];

							//this.SaveInstanceFile( docInstance.Id, version.Id, docInstance.XMLDocumentFile);
							this.SaveAttachFiles (docInstance.Id,version.Id, version.XMLDocumentAttachFiles);
						}

						break;
				}
			}
			catch ( System.IO.IOException )
			{
				throw new Exception.DMDBException("Impossible to save Document Instance");
			}
		}

		public void SaveAttachFiles(string sDocId, string sVerId, XMLDocumentAttachFiles docFiles)
		{
			XMLDocumentFile file = null;
			for ( int i=0; i<docFiles.Count;i++)
			{
				file = (XMLDocumentFile)docFiles[i];

				//this.SaveInstanceFile ( sDocId, sVerId, file);
			}
		}

		#endregion
		
		//retrieve a document template.
		public XMLDocumentTemplate GetDocumentTemplate(string sDocumentName)
		{
			//
			string sDocPath  = this.sDBDirectory + DocumentManagerDBHelper.sDocTemplateDirectory + sDocumentName + ".xml";
			XMLDocumentTemplate document = new XMLDocumentTemplate();
			try
			{
				document.Load(sDocPath);
			}
			catch ( XmlException)
			{
                throw new Scenario.Modules.DM.DataModel.Exception.DocumentNotFound(sDocumentName);
			}

			return document;
		}


		public XMLDocumentInstance GetDocumentInstance(string sDocumentId)
		{
			/*bOS.DataBase.bOSTable xmlDocumentInstance = new XMLDocumentInstance ();
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_ATTRIBUTES_DOCUMENT);
			myCommand.Parameters.Add("@id",sDocumentId);
			this.ReadFromDataBase(myCommand, ref xmlDocumentInstance);

			//
			*/
			string sDocPath  = this.sDBDirectory + DocumentManagerDBHelper.sDocInstanceDirectory;
			XMLAttributesHelper xmlAttributes = this.LoadDocumentAttributes(sDocumentId, "1" );
			string sStorage = string.Empty;
			if ( xmlAttributes != null)
			{
				try
				{
                    Scenario.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttribute xmlAttribute = xmlAttributes.Get("DocumentStorage");
					if ( xmlAttribute != null) 
					{
						sStorage = xmlAttribute.Value;
					}
				}
                catch (Scenario.WorkflowEngine.DataModel.Exception.AttributeNotFound)
				{
				}
			}
			sDocPath+=sStorage;
			sDocPath+=sDocumentId + ".xml";
			
			XMLDocumentInstance document = new XMLDocumentInstance();
			try
			{
				document.Load(sDocPath);
			}
			catch ( XmlException)
			{
				throw new Exception.DMDBException(sDocumentId);
			}

			//document.AddAttribute("DocumentStorage", ((XMLDocumentInstance)xmlDocumentInstance).GetAttribute("DocumentStorage"));
			

			return document;
		}


		public XMLDocuments GetDocuments ( string sWhere ) 
		{
			string sSQL = DocumentManagerDBHelper.SELECT_DOCUMENT_LIST;
			XMLDocuments xmlDocs = new XMLDocuments(false);
			if ( sWhere.CompareTo ( string.Empty ) == 0 )
			{
				sSQL=sSQL.Replace ("#WHERE#", string.Empty );
			}
			else
			{
				sSQL=sSQL.Replace ("#WHERE#", "where " + sWhere);
			}

			MySqlCommand myCommand = this.GetCommand (sSQL );
			
			MySqlDataReader reader = myCommand.ExecuteReader();

			XMLDocumentInstance xmlDoc = null;
			while ( reader.Read() ) 
			{
				xmlDoc=new XMLDocumentInstance();

				xmlDoc.Id = reader.GetValue(0).ToString();
				xmlDoc.Type=reader.GetValue(1).ToString();
				xmlDoc.Name=reader.GetValue(2).ToString();
				xmlDoc.CreationPolicy=reader.GetValue(3).ToString();
				xmlDoc.ManageVersion=reader.GetValue(4).ToString();
				xmlDoc.IdMaster=reader.GetValue(5).ToString();
				xmlDoc.VersionMaster=reader.GetValue(6).ToString();
				xmlDoc.LastDocument=reader.GetValue(7).ToString();
				xmlDoc.XMLDocumentMigrates.Parse(reader.GetValue(8).ToString());
				xmlDoc.XMLGrants.Parse(reader.GetValue(9).ToString());
			//	xmlDoc.AddAttribute ( "DocumentStorage", reader.GetString (10));
			
				xmlDocs.Add (xmlDoc);
			}

			reader.Close();
			myCommand.Connection.Close();

			return xmlDocs;
		}


		public XMLDocuments GetDocumentsTemplate () 
		{
			XMLDocuments xmlDocs = new XMLDocuments(true);

			//TODO: Leggere i DocumentTemplate dal filesystem
			return xmlDocs;
		}



		#region Loading Methods
		public XMLDocumentFile LoadDocumentFile(string sDocumentFileId)
		{
			bOS.DataBase.bOSTable xmlDocumentFile = new XMLDocumentFile ();
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_FILE);
            myCommand.Parameters.AddWithValue("@id", sDocumentFileId);
			this.ReadFromDataBase(myCommand, ref xmlDocumentFile);
			return (XMLDocumentFile)xmlDocumentFile;
		}

		public XMLDocumentInstance LoadDocumentInstance(string sDocumentInstanceId, bool bVersions)
		{
			bOS.DataBase.bOSTable docInstance = new XMLDocumentInstance();

			//modified by e.leano for new version of DM: step 2
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_ATTRIBUTES_DOCUMENT);
            myCommand.Parameters.AddWithValue("@id", sDocumentInstanceId);
			this.ReadFromDataBase(myCommand, ref docInstance);

			//retrieving File
			((XMLDocumentInstance)docInstance).XMLDocumentFile = this.LoadDocumentFile (sDocumentInstanceId);

			if ( bVersions )
			{
				((XMLDocumentInstance)docInstance).XMLDocumentVersions=this.LoadVersions(sDocumentInstanceId);
			}
			
			((XMLDocumentInstance)docInstance).XMLAttributes = this.LoadDocumentAttributes (sDocumentInstanceId, string.Empty);
			
			return (XMLDocumentInstance)docInstance;
		}

		public XMLAttributesHelper LoadDocumentAttributes(string sDocumentId, string sDocumentVersion)
		{
			bOS.DataBase.bOSTable xmlAttHelper = new XMLAttributesHelper();
			MySqlCommand myCommand = new MySqlCommand(DocumentManagerDBHelper.SELECT_ATTRIBUTES);
            myCommand.Parameters.AddWithValue("@id", sDocumentId);
            myCommand.Parameters.AddWithValue("@ver", sDocumentVersion);
			this.ReadFromDataBase(myCommand,ref xmlAttHelper);

			return (XMLAttributesHelper)xmlAttHelper;
		}

		public XMLDocumentVersion LoadDocumentVersion (string sDocumentInstanceId, int iVersion)
		{
			bOS.DataBase.bOSTable xmlDocumentVersion = new XMLDocumentVersion ();

			//modified by e.leano for new version of DM: step 2
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_VERSION_I);
            myCommand.Parameters.AddWithValue("@id", sDocumentInstanceId);
            myCommand.Parameters.AddWithValue("@version", iVersion);
			this.ReadFromDataBase(myCommand, ref xmlDocumentVersion);

			bOS.DataBase.bOSTable idColumn = new bOS.DataBase.StringColumn();
			//modified by e.leano for new version of DM: step 2
			myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_CHILDS);
            myCommand.Parameters.AddWithValue("@id", sDocumentInstanceId);
            myCommand.Parameters.AddWithValue("@version", iVersion.ToString());
			this.ReadFromDataBase(myCommand, ref idColumn);

			foreach ( bOS.DataBase.StringField field in (bOS.DataBase.StringColumn)idColumn)
			{
			
				XMLDocumentInstance xmlDocumentInstance = this.LoadDocumentInstance (field.Id, false);
				((XMLDocumentVersion)xmlDocumentVersion).XMLDocuments.Add (xmlDocumentInstance);
			}
			bOS.DataBase.bOSTable xmlAttHelper = new XMLAttributesHelper();
			myCommand = new MySqlCommand(DocumentManagerDBHelper.SELECT_ATTRIBUTES);
            myCommand.Parameters.AddWithValue("@docId", sDocumentInstanceId);
            myCommand.Parameters.AddWithValue("@ver", iVersion.ToString());
			this.ReadFromDataBase(myCommand,ref xmlAttHelper);
			
			((XMLDocumentVersion)xmlDocumentVersion).XMLAttributes = (XMLAttributesHelper)xmlAttHelper;
			return (XMLDocumentVersion)xmlDocumentVersion;
		}

		public XMLDocumentVersions LoadVersions(string sDocumentInstanceId)
		{
			bOS.DataBase.bOSTable idColumn = new bOS.DataBase.IntegerColumn();
			//modified by e.leano for new version of DM: step 2
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_VERSION);
            myCommand.Parameters.AddWithValue("@id", sDocumentInstanceId);
					
			this.ReadFromDataBase(myCommand, ref idColumn); 

			XMLDocumentVersions xmlVersions = new XMLDocumentVersions();
			foreach ( bOS.DataBase.IntegerField field in (bOS.DataBase.IntegerColumn)idColumn)
			{
				XMLDocumentVersion xmlVersion = this.LoadDocumentVersion (sDocumentInstanceId, field.Id);

				xmlVersions.Add (xmlVersion);
			}
			return xmlVersions;
		}

		#endregion
		
		#region Delete Methods
		public ArrayList DeleteAllChild(string sDocumentId)
		{
			
			ArrayList childId = new ArrayList();
			bOS.DataBase.bOSTable idColumn = new bOS.DataBase.StringColumn();
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.SELECT_ALL_CHILDS);
            myCommand.Parameters.AddWithValue("@id", sDocumentId);
			this.ReadFromDataBase(myCommand, ref idColumn);
			
			foreach ( bOS.DataBase.StringField field in (bOS.DataBase.StringColumn)idColumn)
			{
				childId.Add(field.Id);	
				 
				
				//XMLDocumentInstance xmlDocumentInstance = this.LoadDocumentInstance (field.Id, false);
				//((XMLDocumentVersion)xmlDocumentVersion).XMLDocuments.Add (xmlDocumentInstance);
			}
			myCommand = new MySqlCommand(DocumentManagerDBHelper.DELETE_CHILDS);
            myCommand.Parameters.AddWithValue("@id", sDocumentId);
			this.ExecuteNoQuery(myCommand);
			return childId;


		}

		public void DeleteDocs(ArrayList docIds)
		{
			foreach (string sId in docIds)
				this.DeleteDoc(sId);
		}

		public void DeleteDoc(string sDocumentId)
		{
			MySqlCommand myCommand= new MySqlCommand(DocumentManagerDBHelper.DELETE_DOC);
            myCommand.Parameters.AddWithValue("@id", sDocumentId);
			this.ExecuteNoQuery(myCommand);
			myCommand= new MySqlCommand(DocumentManagerDBHelper.DELETE_FILE);
            myCommand.Parameters.AddWithValue("@id", sDocumentId);
			this.ExecuteNoQuery(myCommand);
			myCommand= new MySqlCommand(DocumentManagerDBHelper.DELETE_VER);
            myCommand.Parameters.AddWithValue("@id", sDocumentId);
			this.ExecuteNoQuery(myCommand);
			
		}

		#endregion
	}
}
