using System;
using System.Collections;
using System.Configuration;

namespace DirectoryWatcher
{
    /// <summary>
    /// Collection of filesystem events to use when monitoring file changes.
    /// </summary>
    public class EventToWatchCollection : ConfigurationElementCollection
    {
        /// <summary>
        /// Numeric indexer for the collection.
        /// </summary>
        /// <param name="index">
        /// Index in the collection whose contents we are to return.
        /// </param>
        /// <returns>
        /// EventToWatch object at the specified index.
        /// </returns>
        public EventToWatch this[int index]
        {
            get
            {
                return (EventToWatch)base.BaseGet(index);
            }
        }

        /// <summary>
        /// String indexer for the collection; uses the Type property of each EventToWatch object
        /// as the index.
        /// </summary>
        /// <param name="key">
        /// Value of the Type property of the EventToWatch object that we are to return.
        /// </param>
        /// <returns>
        /// EventToWatch object with the specified type.
        /// </returns>
        public new EventToWatch this[string key]
        {
            get
            {
                return (EventToWatch)base.BaseGet(key);
            }
        }

        /// <summary>
        /// Tests to see whether a given element name is a member of this collection.
        /// </summary>
        /// <param name="elementName">
        /// Element name that we are to test.
        /// </param>
        /// <returns>
        /// True of the element name is part of this collection, false otherwise.
        /// </returns>
        protected override bool IsElementName(string elementName)
        {
            if ((string.IsNullOrEmpty(elementName)) || (elementName != "eventToWatch"))
                return false;

            return true;
        }

        /// <summary>
        /// Collection type of this collection.
        /// </summary>
        public override ConfigurationElementCollectionType CollectionType
        {
            get
            {
                return ConfigurationElementCollectionType.BasicMapAlternate;
            }
        }

        /// <summary>
        /// Gets value to use as the key for a specific element in this collection.
        /// </summary>
        /// <param name="element">
        /// Element whose key we are to retrieve.
        /// </param>
        /// <returns>
        /// The Type property of the specified EventToWatch object.
        /// </returns>
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((EventToWatch)element).Type.ToString();
        }

        /// <summary>
        /// Creates a new element to add to the collection.
        /// </summary>
        /// <returns>
        /// A new EventToWatch object.
        /// </returns>
        protected override ConfigurationElement CreateNewElement()
        {
            return new EventToWatch();
        }
    }
}
